/*=========================================================================
*
* MatStack.c	- matrix stack
*
* Version			1.0
* Revision date
*
*=======================================================================*/
#include "matstack.h"



//--- Global defines ---------------------------------------------------------

MLT_tdstMatrix *g_p_stCurrentMatrix ;


//--- Global Statics ---------------------------------------------------------

MLT_tdstMatrix	gs_aDEF_stMatrixStack [ MLT_C_lMaxMatrixInStack ] ;
long			gs_lNbMatrixInStack ;


//--------------------------------------------------------------------



/****************************************************************************
*		Description:	init the matrix stack with the identity matrix 
*
*		Parameters:		
*---------------------------------------------------------------------------
*		Revision date:                                      Author:         
*****************************************************************************/
void MLT_xInitMatrixStack (void)
{
  gs_lNbMatrixInStack = 0;

  MLT_xSetIdentityMatrix(&gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack ]);
  
  g_p_stCurrentMatrix = &gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack ] ;

  gs_lNbMatrixInStack ++;
}

/****************************************************************************
*		Description:	reset the matrix stack to NULL
*
*		Parameters:		
*---------------------------------------------------------------------------
*		Revision date:                                      Author:         
*****************************************************************************/
void MLT_xResetMatrixStack ( void )
{
  gs_lNbMatrixInStack = 0 ;

  g_p_stCurrentMatrix = NULL ;
}



/****************************************************************************
*		Description:	pop the last matrix from the matrix stack
*
*		Parameters:		
*---------------------------------------------------------------------------
*		Revision date:                                      Author:         
*****************************************************************************/
void MLT_xPopMatrix ( void )
{
  if ( gs_lNbMatrixInStack == 0 )
    return ;

  gs_lNbMatrixInStack -- ;

  g_p_stCurrentMatrix = &gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack ] ;
}


/****************************************************************************
*		Description:	push the given matrix on the matrix stack 
*
*		Parameters:		p_stMatrix	: source matrix
*---------------------------------------------------------------------------
*		Revision date:                                      Author:         
*****************************************************************************/
void MLT_xPushMatrix ( MLT_tdstMatrix *p_stMatrix )
{
  if ( gs_lNbMatrixInStack == MLT_C_lMaxMatrixInStack )
    return;

  MLT_xMulMatrixMatrix ( &gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack ] ,
                         &gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack - 1 ] ,
                         p_stMatrix ) ;

  g_p_stCurrentMatrix = &gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack ] ;

  gs_lNbMatrixInStack ++ ;
}

/****************************************************************************
*		Description:	pre-push the given matrix on the matrix stack 
*
*		Parameters:		p_stMatrix	: source matrix
*---------------------------------------------------------------------------
*		Revision date:                                      Author:         
*****************************************************************************/
void MLT_xPrePushMatrix ( MLT_tdstMatrix *p_stMatrix )
{
  if ( gs_lNbMatrixInStack == MLT_C_lMaxMatrixInStack )
    return;

  MLT_xMulMatrixMatrix ( &gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack ] ,
                         p_stMatrix ,
                         &gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack - 1 ] );

  g_p_stCurrentMatrix = &gs_aDEF_stMatrixStack [ gs_lNbMatrixInStack ] ;

  gs_lNbMatrixInStack ++ ;
}


