/*=========================================================================
* PlayEvts.h : This module contain all functions used to play events
* This is a part of the Game project.
*
* Version              1.0
* Creation date        6/11/96
* Revision date
*
* That file needs to be compatible for all platforms.
*
* (c) Ubi Studios 1996
*=======================================================================*/
 
#if !defined(__PLAYEVTS_H__)
#define __PLAYEVTS_H__
 
/******************************************/
#if _MSC_VER >= 1000
#pragma once
#endif /* _MSC_VER >= 1000*/
/******************************************/
 
#include "GAM/Header.h"
 
#define C_ucNormal  0
#define C_ucReverse 1
#define C_ucStopEvent 2
 
void fn_vInitAllEvents(HIE_tdxHandleToSuperObject p_stSuperObject,unsigned short uwFirstFrame);
void fn_vDesInitAllEvents(HIE_tdxHandleToSuperObject p_stSuperObject,tdxHandleToState hNextState,BOOL bStopEvents);
void fn_vPlayEventsSinceLastOneNoLoop(HIE_tdxHandleToSuperObject p_stSuperObject );
void fn_vPlayLastEventsBeforeRestart(HIE_tdxHandleToSuperObject p_stSuperObject );
void fn_vPlayEvent( struct tdstEvent_ * p_stEvent, unsigned char ucMode ,HIE_tdxHandleToSuperObject p_stSuperObject);
unsigned short fn_uwGetNbEngineFrameSinceLastMechEvent(HIE_tdxHandleToSuperObject p_stSuperObject);
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the type of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *  Output :        the type
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
unsigned char GAM_fn_ucGetTypeOfEvent(tdstEvent* _p_stEvent);
 
tdstEvent *fn_p_GetEventsAddress(tdstAnim3d *_p_stAnim3d, 
								 unsigned long	_ulFrame,
								 tdxHandleToObjectsTablesList _h_ObjectsTablesListElement); /*AR9809 Parameter necessary for U64*/
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the pointed event of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *  Output :        the type
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
void* GAM_fn_p_xGetEventOfEvent(tdstEvent* _p_stEvent);
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the address pointed event of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *  Output :        the address
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
void** GAM_fn_p_xGetAdressOfEventOfEvent(tdstEvent* _p_stEvent);
 
/*-----------------------------------------------------------------------------
 *  Description :   Set the pointed event of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the animation event
 *					_p_xRealEvent : pointed event
 *  Output :        
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
void GAM_fn_vSetEventOfEvent(tdstEvent* _p_stEvent,void* _p_xRealEvent);
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the first call field of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *  Output :        the first call
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
unsigned char GAM_fn_ucGetFirstCallOfEvent(tdstEvent* _p_stEvent);
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the first call field of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *					_ucFirstCall : The first call field
 *  Output :        
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
void GAM_fn_vSetFirstCallOfEvent(tdstEvent* _p_stEvent,unsigned char _ucFirstCall);
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the period field of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *  Output :        the period
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
unsigned char GAM_fn_ucGetPeriodOfEvent(tdstEvent* _p_stEvent);
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the first call field of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *					_ucPeriod : The period
 *  Output :        
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
void GAM_fn_vSetPeriodOfEvent(tdstEvent* _p_stEvent,unsigned char _ucPeriod);
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the binary event id field of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *  Output :        the period
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
unsigned long GAM_fn_ulGetBinaryEventIdOfEvent(tdstEvent* _p_stEvent);
 
/*-----------------------------------------------------------------------------
 *  Description :   Get the first call field of an event (V5 or V6 format)
 *-----------------------------------------------------------------------------
 *  Input :         _p_stEvent : pointer to the event
 *					_ulBinaryEventId : The binary event id
 *  Output :        
 *-----------------------------------------------------------------------------
 *  Creation date : Jan 98            Author : Alain Robin
 *---------------------------------------------------------------------------*/
void GAM_fn_vSetBinaryEventIdOfEvent(tdstEvent* _p_stEvent,unsigned long _ulBinaryEventId);
 
#endif /* __PLAYEVTS_H__ */
