/*///////////////////////////////////////////////////////////*/
/*															//*/
/*			Management of the Module : GMT					//*/
/*															//			   */
/* File Name	: ErrGmt.h									//*/
/* Date			: 07/10/97									//*/
/* Author		: Alain Robin								//*/
/*															//*/
/*///////////////////////////////////////////////////////////*/
/*															//*/
/* abbreviation of the module-name. Used in macro is 'Gmt'	//*/
/*															//*/
/*///////////////////////////////////////////////////////////*/
 
#ifndef __ERRGMT_H__
#define __ERRGMT_H__
 
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
 
#define C_szGmtVersion "GMT V5.0.0" /* The format is XXX Va.b.c with Xxx is the Tag of the module */
#define C_szGmtFullName  "Game material module"/* the complete and clear name of the module */
#define C_szGmtDate __DATE__ /*The format is "Mmm dd yyyy".You can use __DATE__ but be careful that you have the control of the compilation*/
 
/* For DLLs who are using this module : */ 
#undef CPA_EXPORT
#if defined(CPA_WANTS_IMPORT)
 #define CPA_EXPORT __declspec(dllimport)
#elif defined(CPA_WANTS_EXPORT)
 #define CPA_EXPORT __declspec(dllexport)
#else
 #define CPA_EXPORT
#endif
 
#include "ERM.h"
 
#define __FATAL_ERR_GMT__
#define __WARNING1_ERR_GMT__
#define __WARNING2_ERR_GMT__
#define __WARNING3_ERR_GMT__
/*----------*/
/* Constant	 */
/*----------*/
/* error of the Gmt Module*/
typedef enum e_uwGmtErrNumber_{
	E_uwGmtFatalErr,	
#ifdef __FATAL_ERR_GMT__
/*     -----------------*/
	E_uwGmtFatalError1,	 
	E_uwGmtFatalError2,	
	E_uwGmtFatalError3,	
	E_uwGmtFatalError4,	
#endif /*__FATAL_ERR_GMT__*/
	E_uwGmtStartOfWarning,/* important constant, it allows to recognize if an error is fatal or not.*/
#ifdef __WARNING1_ERR_GMT__
/*     -------------------*/
	E_uwGmtWarning1Error1,	
	E_uwGmtWarning1Error2,	
	E_uwGmtWarning1Error3,
#endif /*__WARNING1_ERR_GMT__*/
#ifdef __WARNING2_ERR_GMT__
/*     -------------------*/
	E_uwGmtWarning2Error1,	
	E_uwGmtWarning2Error2,	
	E_uwGmtWarning2Error3,	
	E_uwGmtWarning2Error4,	
	E_uwGmtWarning2Error5,
#endif /*__WARNING2_ERR_GMT__*/
#ifdef __WARNING3_ERR_GMT__
/*     -------------------*/
	E_uwGmtWarning3Error1,	
	E_uwGmtWarning3Error2,
#endif /*__WARNING3_ERR_GMT__*/
	E_uwGmtErrNumber
} e_uwGmtErrNumber;
 
/*------------------*/
/* Global Variables*/
/*------------------*/
#undef __ERRGMT_EXTERN
#ifndef __DeclareGlobalVariableErmGmt_h__
#define __ERRGMT_EXTERN extern /*external declaration*/
#else /*__DeclareGlobalVariableErmGmt_h__*/
#define __ERRGMT_EXTERN /*replace by nothing : we have to declare*/
#endif /*__DeclareGlobalVariableErmGmt_h__*/
 
#ifndef _FIRE_DEADCODE_U64_ /* Added by RUC */
__ERRGMT_EXTERN CPA_EXPORT  unsigned char g_ucGmtModuleId /*number of identification of the Erm module*/
#if defined(__DeclareGlobalVariableErmGmt_h__) && !defined(CPA_WANTS_IMPORT)
= C_ucModuleNotInitialized
#endif /*__DeclareGlobalVariableErrGmt_h__&& CPA_WANTS_IMPORT*/
;
#endif /* _FIRE_DEADCODE_U64_ */ /* Added by RUC */
 
#ifdef __ERROR_STRINGS__
 
__ERRGMT_EXTERN CPA_EXPORT  char * g_a_szGmtInformationModule []
#if defined(__DeclareGlobalVariableErmGmt_h__) && !defined(CPA_WANTS_IMPORT)
= {C_szGmtVersion, C_szGmtFullName, C_szGmtDate}
#endif /*__DeclareGlobalVariableErrGmt_h__ &&  CPA_WANTS_IMPORT*/
;
 
__ERRGMT_EXTERN CPA_EXPORT  char * g_szGmtModuleName /* Mandatory syntax 'g_sz'+[Abbreviation of ModuleName]+'ModuleName'*/
#if defined(__DeclareGlobalVariableErmGmt_h__) && !defined(CPA_WANTS_IMPORT)
= "Module Gmt"
#endif /*__DeclareGlobalVariableErrGmt_h__ && CPA_WANTS_IMPORT*/
;
 
__ERRGMT_EXTERN CPA_EXPORT  struct tdstErrorMsg_ g_a_stGmtTabErr [] 	/* Mandatory syntax 'g_a_st'+[Abbreviation of ModuleName]+'TabErr'*/
#if defined(__DeclareGlobalVariableErmGmt_h__) && !defined(CPA_WANTS_IMPORT)
={ 
#ifdef __FATAL_ERR_GMT__   
/*	 -------------*/
	E_uwGmtFatalError1, " The handle of the game material is invalid ",
	E_uwGmtFatalError2, " ",
	E_uwGmtFatalError3, " ",
	E_uwGmtFatalError4, " ",
#endif /*__FATAL_ERR_GMT__*/
#ifdef __WARNING1_ERR_GMT__
/*	 ----------------*/
	E_uwGmtWarning1Error1, " ",	
	E_uwGmtWarning1Error2, " ",
	E_uwGmtWarning1Error3, " ",
#endif /*__WARNING1_ERR_GMT__*/
#ifdef __WARNING2_ERR_GMT__
/*	 ----------------*/
	E_uwGmtWarning2Error1, " ",
	E_uwGmtWarning2Error2, " ",
	E_uwGmtWarning2Error3, " ",
	E_uwGmtWarning2Error4, " ",
	E_uwGmtWarning2Error5, " ",
#endif /*__WARNING2_ERR_GMT__*/
#ifdef __WARNING3_ERR_GMT__
/*	 ----------------*/
	E_uwGmtWarning3Error1, " ",
	E_uwGmtWarning3Error2, " ",
#endif /*__WARNING3_ERR_GMT__*/
	0xFFFF, "\0"/*fin*/
};
#endif /*__DeclareGlobalVariableErrGmt_h__ && CPA_WANTS_IMPORT*/
;
 
#endif /*__ERROR_STRINGS__*/
 
/* ////////////////////////////////////////////////////////////////////////////
//  Description : GMT_fn_vRaiseFatalError
//  Used to raise a fatal error of the GMT module (PRIVATE FUNCTION !)
//////////////////////////////////////////////////////////////////////////////
//  Input :  
//  Output :    
//////////////////////////////////////////////////////////////////////////// */
void GMT_fn_vRaiseFatalError();
 
#ifdef __cplusplus
};
#endif /* __cplusplus */
 
 
#endif /*__ERRGMT_H__*/
