/*
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *  SCR_XXXX.h
 *  Compatibility file with version 1.8.X of script parser.
 *
 *  Scripts, Beaudet Christophe
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *
 */
 
#ifndef __SCR_XXXX_h__
#define __SCR_XXXX_h__
 
/*
 *=================================================================================================
 *  Constants.
 *=================================================================================================
 */
 
/*
 *  Actions in callbacks.
 */
#define C_SCRIPT_AlreadyAnalysed                SCR_EA_Anl_AlreadyAnalysed
#define C_SCRIPT_Section                        SCR_EA_Anl_BeginSection
#define C_SCRIPT_BegSubSection                  SCR_EA_Anl_BeginSubSection
#define C_SCRIPT_EndSection                     SCR_EA_Anl_EndSection
#define C_SCRIPT_EndSubSection                  SCR_EA_Anl_EndSubSection
#define C_SCRIPT_Entry                          SCR_EA_Anl_Entry
 
/*
 *  Global values.
 */ 
#define C_SCRIPT_uiMaxLenID                     SCR_CV_Cfg_MaxLenWord
#define C_SCRIPT_uiMaxLenLine                   SCR_CV_Cfg_MaxLenLine
#define C_SCRIPT_cRefSubSection                 SCR_CC_c_Cfg_NameSeparator
#define C_SCRIPT_szRefSubSection                SCR_CC_sz_Cfg_NameSeparator
 
/*
 *  Return values.
 */
#define R_SCRIPT_NormalReturn                   SCR_ERV_Anl_NormalReturn
#define R_SCRIPT_TerminateCurrentSection        SCR_ERV_Anl_TerminateCurrentSection
#define R_SCRIPT_TerminateFile                  SCR_ERV_Anl_TerminateFile
#define R_SCRIPT_TerminateMainSection           SCR_ERV_Anl_TerminateMainSection
 
/*
 *  Format to save.
 */
#define F_SCRIPT_NoFormat	                    SCR_EF_SvL0_Normal
#define F_SCRIPT_ArrayByte	                    SCR_EF_SvL0_ArrayByte
#define F_SCRIPT_ArrayShort	                    SCR_EF_SvL0_ArrayShort
#define F_SCRIPT_ArrayLong	                    SCR_EF_SvL0_ArrayLong
#define F_SCRIPT_ArrayFloat	                    SCR_EF_SvL0_ArrayFloat
#define F_SCRIPT_ArrayDouble	                SCR_EF_SvL0_ArrayDouble
#define F_SCRIPT_ArrayInt	                    SCR_EF_SvL0_ArrayInt
#define F_SCRIPT_ArrayBoolean	                SCR_EF_SvL0_ArrayBoolean
#define F_SCRIPT_ArrayDisEna	                SCR_EF_SvL0_ArrayDisEna
#define F_SCRIPT_ArrayReference	                SCR_EF_SvL0_ArrayReferences
#define F_SCRIPT_Scanf	                        SCR_EF_SvL0_Scanf
 
/*
 * For hash and link.
 */
#define S_REF_Valid                             SCR_ELS_Link_NotInitialized
#define S_REF_Free                              SCR_ELS_Link_Initialized
 
/*
 *=================================================================================================
 *  Macros.
 *=================================================================================================
 */
#define SCRIPT_PCHAR(_Exp)                      SCR_M_RdL0_PCHAR(_Exp)
#define SCRIPT_CHAR(_Exp)                       SCR_M_RdL0_CHAR(_Exp)
#define SCRIPT_PSHORT(_Exp)                     SCR_M_RdL0_PSHORT(_Exp)
#define SCRIPT_SHORT(_Exp)                      SCR_M_RdL0_SHORT(_Exp)
#define SCRIPT_PLONG(_Exp)                      SCR_M_RdL0_PLONG(_Exp)
#define SCRIPT_LONG(_Exp)                       SCR_M_RdL0_LONG(_Exp)
#define SCRIPT_PINT(_Exp)                       SCR_M_RdL0_PINT(_Exp)
#define SCRIPT_INT(_Exp)                        SCR_M_RdL0_INT(_Exp)
#define SCRIPT_PFLOAT(_Exp)                     SCR_M_RdL0_PFLOAT(_Exp)
#define SCRIPT_FLOAT(_Exp)                      SCR_M_RdL0_FLOAT(_Exp)
#define SCRIPT_PDOUBLE(_Exp)                    SCR_M_RdL0_PDOUBLE(_Exp)
#define SCRIPT_DOUBLE(_Exp)                     SCR_M_RdL0_DOUBLE(_Exp)
 
/*
 *=================================================================================================
 *  Types.
 *=================================================================================================
 */
#define SCRIPT_tdeAction                        SCR_tde_Anl_Action
#define SCRIPT_tdeReturnValue                   SCR_tde_Anl_ReturnValue
#define SCRIPT_tdstSaveFile                     SCR_tdst_File_Description
#define REF_tdstLinkTable                       SCR_tdst_Link_Table
#define REF_tdxLinkId                           SCR_tdst_Link_Value *
 
/*
 *=================================================================================================
 *  Functions (macros).
 *=================================================================================================
 */
 
#define SCRIPT_vFirstInit()                     SCR_fn_v_RdL0_Init()
#define SCRIPT_vLastClose()                     SCR_fn_v_RdL0_Close()
 
#define SCRIPT_p_stGetLastError()               SCR_fnp_st_RdL0_GetLastError()
#define SCRIPT_vClearError()                    SCR_fn_v_RdL0_ClearError()
 
#define SCRIPT_vDelAllOpenSections              SCR_fn_v_RdL0_DeleteOpenSection("", SCR_CDR_c_RdL0_Contains)
#define SCRIPT_vDelOpenSection(a)               SCR_fn_v_RdL0_DeleteOpenSection(a, SCR_CDR_c_RdL0_Contains)
 
#define SCRIPT_vRegisterPath(a)                 SCR_fn_v_RdL0_RegisterPath(a)
#define SCRIPT_vDelAllPathNames()               SCR_fn_v_RdL0_DeleteRegisterPath("", SCR_CDR_c_RdL0_Contains)
 
#define SCRIPT_vRegisterSection(a, b)           SCR_fn_v_RegisterCallback(a, (SCR_tdpfn_Anl_Callback) b, SCR_CRC_c_RdL0_ForSection)
#define SCRIPT_p_stGetRegisterSection(a)        SCR_fn_p_st_RdL0_GetRegisterCallback(a, SCR_CRC_c_RdL0_ForSection)
#define SCRIPT_vDelAllRegSections()             SCR_fn_v_RdL0_DeleteRegisterCallback("", SCR_CDR_c_RdL0_Contains)
#define SCRIPT_vDelRegisterSection(a)           SCR_fn_v_RdL0_DeleteRegisterCallback(a, SCR_CDR_c_RdL0_Contains)
 
#define SCRIPT_ucGetNumberOfArgs(a)             SCR_fn_uc_RdL0_GetNumberOfParameters(a)
#define SCRIPT_vSplitSectionName(a, b, c)       SCR_fn_v_RdL0_SplitSectionName(a, NULL, b, c)
 
#define SCRIPT_p_szGetFileName(a)               SCR_M_RdL0_GetOpenFileNameR(a)
#define SCRIPT_p_szGetSectionName(a)            SCR_M_RdL0_GetCompleteSectionNameR(a)
 
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
#define REF_fn_uiComputeHashKeyForString(a)         SCR_fn_x_Hash_ComputeHashKeyForString(a)
#define REF_fn_vInitLinkTable(a)                    SCR_fn_v_Link_InitTable(a)
#define REF_fn_vDelReallyLinkTable(a)               SCR_fn_v_Link_CloseTable(a)
#define REF_fn_vAddEntryToLinkTable(a, b, c, d, e)  SCR_fnp_st_Link_AddEntryToLinkTable(a, b, d, e)
#define REF_fn_xSearchKeyInLinkTable(a, b, c)       SCR_fnp_st_Link_SearchKey(a, b)
#define REF_fn_xCreateOrGetIdFromKey(a, b, c)       SCR_fnp_st_Link_CreateOrGetLinkFromKey(a, b)
#define REF_fn_xSetValue(a, b, c, d)                SCR_fnp_st_Link_SetValue(a, b, d)
#define REF_fn_ulGetValueFromId(a, b)               ((b)->ulValue)
#define REF_fn_uiGetHashKeyFromId(a, b)             ((b)->xHashKeyForKey)
#define REF_fn_eGetStateFromId(a, b)                ((b)->eState)
#define REF_fn_xGetIdFromValue(a, b)                SCR_fnp_st_Link_SearchValue(a, b)
#define REF_fnp_szGetKeyFromId(a, b)                ((b)->szKey)
#define REF_fn_vInvalidateMyObject(a, b)            {*(b) = NULL;}
#define REF_fn_cIsMyObjectValid(a, b)               ((b) != NULL)
 
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
#define SCRIPT_vSaveEndSection(a)               SCR_M_SvL0_SaveEndSection(a, SCR_CC_C_Cfg_EOL)
#define SCRIPT_vSaveCommentLine(a, b)           SCR_M_SvL0_SaveComment(a, b)
#define SCRIPT_vSaveEmptyLine(a)                SCR_M_SvL0_SaveBlankLine(a)
#define SCRIPT_vSaveCharacters(a, b)            SCR_fn_v_SvL0_SaveWord(a, SCR_CC_C_Cfg_NoChar, b, SCR_CC_C_Cfg_NoChar)
#define SCRIPT_uiGetSaveIndentLevel(void)       (SCR_g_ui_SvL0_IndentationLevel)
#define SCRIPT_vSetSaveIndentLevel(a)           {SCR_g_ui_SvL0_IndentationLevel = (a);}
 
#define SCRIPT_vSaveBeginSection(a, b, c, d, e, f)
#define SCRIPT_vSaveEntryInSection(a, b, c, d, e, f)
#define SCRIPT_vSaveFileDescription(a, b, c)
#define SCR_fn_v_RdL0_DelOpenFile(a, b)			SCR_fn_v_RdL0_DeleteOpenFile(a, b)
 
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/* SCR V5.4.1 */
#define SCR_fn_v_RdL0_InitWithMem(a)				SCR_fn_v_RdL0_Init()
#define SCR_fn_v_RdL0_ReduceMemory()				SCR_fn_v_Mem_ReduceMemory()
#define SCR_fn_v_RdL0_SetCurrentFilePriority(a)		SCR_fn_v_Mem_SetCurrentLevel(a)
#define SCR_fn_uc_RdL0_GetCurrentFilePriority()		SCR_fn_uc_Mem_GetCurrentLevel()
#define SCR_M_uc_Link_GetPriority(a)				SCR_M_uc_Link_GetMemLevel(a)
#define SCR_fn_v_Link_DeleteEntryWithPriority(a, b, c)
#define SCR_fn_v_RdL0_DeleteOpenWithPriority(a, b)
 
#endif /* __SCR_XXXX_h__ */
