/*
 =======================================================================================
  Name        : DispMode.h
  Author      : vincent lhullier            Date :04/09/98
  Description : Manage the different display mode available
 =======================================================================================
 */
 
#ifndef __DISPMODE_H__
#define __DISPMODE_H__
 
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
#include <stdio.h>
#include "GLI_util.h"
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
#ifdef __cplusplus
extern "C" 
{
#endif

/*
 =======================================================================================
    Structure
 =======================================================================================
 */
 
/*
 * to store display mode
 */
typedef struct GLI_tdstDisplayMode_
{
    long    lFullScreen;
    long    lWidth;
    long    lHeight;
    long    lBpp;
} GLI_tdstDisplayMode;

/*
 =======================================================================================
    Constants
 =======================================================================================
 */
#define GLI_C_lMaxDisplayMode 32

/*
 =======================================================================================
    Globals
 =======================================================================================
 */
extern GLI_tdstDisplayMode  GLI_gs_a_stDisplayMode[ GLI_C_lMaxDisplayMode ];
extern long                 GLI_gs_lNumberOfDisplayModes; 
extern long                 GLI_gs_lCurrentDisplayMode; 
extern long                 GLI_gs_lRequestedDisplayMode;


/*
 =======================================================================================
    Access function
 =======================================================================================
 */
 
/*
 * for display mode
 */
long GLI_fn_lIsFullScreenMode( void );
long GLI_fn_lDisplayModeWidth( void );
long GLI_fn_lDisplayModeHeight( void );
long GLI_fn_lDisplayModeBPP( void );
long GLI_fn_lGetDisplayMode( void );
 
long GLI_fn_lAddDisplayMode( long _lFullScreen, long _lWidth, long _lHeight, long _lBpp );
 
long GLI_fn_lGetNumberOfDisplayModes( void );
long GLI_fn_lGetCurrentDisplayMode( void );
long GLI_fn_lGetDisplayModeParameters( long _lDisplayMode, long *_p_lWidth, long *_p_lHeight );
long GLI_fn_lGetDisplayModeDescription( long _lDisplayMode, char *_szDesc );
 
void GLI_fn_lRequestNewDisplayMode( long _lDisplayMode );
long GLI_fn_lChangeDisplayMode( void );
 
 
/*
 =======================================================================================
    Function
 =======================================================================================
 */
 
 
/*
 * choose / select
 */
long GLI_fn_lSelectDisplayModeByIndex( long _lIndex );
long GLI_fn_lSelectDisplayModeByDimension( long _lWidth, long _lHeight, long _lBpp );
 
/*
 * init
 */
void GLI_fn_vLoadDisplayMode( void );
 
/*
 *
 */
void GLI_fn_vSetBrightness( long _lValue );
long GLI_fn_lGetBrightness();

 
#ifdef __cplusplus
}
#endif
 
 
#endif /*__DISPMODE_H__*/
