//
// Common functions for Object DLL capabilities
// C. Beaudet
//

#include "stdafx.h"
#include "afxdllx.h"
#include "ACP_Base.h"
#include "ITF.h"

#include "3Dinterf.hpp"


//------------------------------------------------------------------------
// Global vars
static char                         *gs_p_szCPAVersion = C_szCPAVersion;
static AFX_EXTENSION_MODULE NEAR    extensionDLL = { NULL, NULL };

#ifdef DLL_ONLY_ONE_INSTANCE
static Geometry3D_Interface *gs_p_oGeometry3DInterface;
#endif

static CList<CPA_DLLBase*,CPA_DLLBase*> g_oListOfInstances;	//private internal

//------------------------------------------------------------------------
// functions that are present in all DLL :
//------------------------------------------------------------------------

//========================================================================
// Get current CPA version
//========================================================================
extern "C" char __declspec(dllexport) *fn_p_szGetCPAVersion(void)
{
    return gs_p_szCPAVersion;
}

//========================================================================
// Get type of this DLL
//========================================================================
extern "C" tdstDLLIdentity __declspec(dllexport) *fn_p_stGetDLLIdentity(void)
{
    g_stGeometry3DIdentity . eType              = OBJECT_DLL; 
    g_stGeometry3DIdentity . csName             = C_szDLLGeometryName;
    g_stGeometry3DIdentity . hModule            = NULL;
    g_stGeometry3DIdentity . p_oListOfInstances = &g_oListOfInstances;

    return &g_stGeometry3DIdentity;
}

//========================================================================
// DLL int function
//========================================================================
extern "C" void __declspec(dllexport) fn_vInitDll(void)
{
	new CDynLinkLibrary(extensionDLL);
}

//========================================================================
// DLL entry point
//========================================================================
extern "C" int __stdcall DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		if (!AfxInitExtensionModule(extensionDLL, hInstance))
		   return 0;
	}
	return 1;
}

//========================================================================
// Get the DLL
//========================================================================
CPA_DLLBase __declspec(dllexport) *fn_p_oGetDLL(long lKey)
{
#ifdef DLL_ONLY_ONE_INSTANCE

	switch(lKey)
	{
	case 0:	// the game world
		if (gs_p_oGeometry3DInterface == NULL)
		{
			gs_p_oGeometry3DInterface = new Geometry3D_Interface();
			ASSERT(gs_p_oGeometry3DInterface != NULL);
		}
		return gs_p_oGeometry3DInterface;
		break;

	default:
		return NULL;
	}

#else //DLL_ONLY_ONE_INSTANCE

	switch(lKey)
	{
	case 0:	// the game world
		return new Geometry3D_Interface();
		break;

	default:
		return NULL;
	}

#endif //DLL_ONLY_ONE_INSTANCE
}


//------------------------------------------------------------------------
// functions that are present in this type of DLL only :
//------------------------------------------------------------------------

#undef DLL_ONLY_ONE_INSTANCE
