/*
///////////////////////////////////////////////////////////////////////////////////////////////////
// Description : LightObj.cpp
//
// Definition of the Light editors objects
//
///////////////////////////////////////////////////////////////////////////////////////////////////
// inherit from  :	CPA_Object
//					CPA_EdMot<ACP_tdxHandleOfLink>
///////////////////////////////////////////////////////////////////////////////////////////////////
// Creation date: 11 feb 1997                        Author: J Thnoz
///////////////////////////////////////////////////////////////////////////////////////////////////
// Modification date:			                      Author:
//
//
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////
*/



#include "stdafx.h"
#include "acp_base.h"
#include "resource.h"
#include "itf.h"
#include "ogd.h"
#include "EdLight.h"
#include "LightObj.hpp"
#include "Inter.hpp"
#include "LightMod.hpp"
#include "gli.h"
#include "LightDia.hpp"
#include "incgam.h"
#include "incsct.h"
#include "dpt.h"
#include "x:\cpa\main\inc\_editid.h"

#include "Col.h"
 
//------------------------------------------------------------------------------------------
// script definitions
//------------------------------------------------------------------------------------------
static char* pszLightAction = "Light";
static char* pszMatrixAction = "Matrix";
static char* pszLightSectorSubSection = "StaticLights";
//------------------------------------------------------------------------------------------


typedef struct Light_Info
{
	Light_Interface*	pInterface;
	CPA_SuperObject*	pSector;
}
Light_Info;


//----------------------------------------------- static variables
DeclareTemplateStatic(GLI_tdxHandleToLight);


//--Static
DiaLight* Light::ms_poDiaLight;
BOOL	Light::ms_bNotifyHeader;
BOOL	Light::ms_bStopNotification;
Light_Interface* Light::ms_poInterface;
CPA_List<CPA_SectionObject>	Light::ms_oListOfSectionObject;



void del(GLI_tdxHandleToLight h)
{
	if (M_GetMainApp()->m_bLeavingApplication)
		return;
	fn_vDestroyLight(h);
}

void Light::fn_vSetDialog ( DiaLight* poDiaLight, Light_Interface* pInterface )
{
	ms_poDiaLight=poDiaLight;
	ms_poInterface=pInterface;
	ms_poDiaLight->fn_vInitDialog (pInterface);
}

void Light::fn_vInitObject (void)
{
	// init Edmot
	CPA_EdMot<GLI_tdxHandleToLight>::Init (fn_hConstructLight, NULL, del);
	ms_bNotifyHeader=FALSE;
}

void Light::fn_vInitListOfSectionObject (void)
{
	ms_oListOfSectionObject.RemoveAll ();
	CPA_List<CPA_SuperObject>* poListOfSector = ms_poInterface->GetInterface()->GetObjectListByType	(C_szSectorTypeName);

	POSITION xSuperObjectPos;
	CPA_SuperObject* poSuperObject;

	for ( poSuperObject=poListOfSector->GetHeadElement(xSuperObjectPos); poSuperObject; poSuperObject=poListOfSector->GetNextElement(xSuperObjectPos) )
	{
			CString csSectorSectionName = ((CPA_SaveObject*)(poSuperObject->GetObject()))->GetReferencedSectionName ();
			csSectorSectionName += "^" ;
			csSectorSectionName += pszLightSectorSubSection;

			CPA_SectionObject* poNewSection = new CPA_SectionObject ( csSectorSectionName.GetBuffer (80), fn_szGetLevelsDataPath(), Light::fn_vCallBackSaveSector, poSuperObject, FALSE );
			ms_oListOfSectionObject.AddTail (poNewSection);
	}
}

void Light::fn_vFreeListOfSectionObject (void)
{
	POSITION xSectionObjectPos;
	CPA_SectionObject* poSectionObject;

	for (poSectionObject=ms_oListOfSectionObject.GetHeadElement(xSectionObjectPos); poSectionObject; poSectionObject=ms_oListOfSectionObject.GetNextElement(xSectionObjectPos) )
		delete poSectionObject;

	ms_oListOfSectionObject.RemoveAll ();
}

CPA_SectionObject* Light::fn_poFindSectionObject ( CPA_SuperObject* poSector )
{
	POSITION xSectionObjectPos;
	CPA_SectionObject* poSectionObject;

	for (poSectionObject=ms_oListOfSectionObject.GetHeadElement(xSectionObjectPos); poSectionObject && poSectionObject->GetSectionData()!=poSector; poSectionObject=ms_oListOfSectionObject.GetNextElement(xSectionObjectPos) );

	// search sector section not found in global list of section sector
	ASSERT (poSectionObject);

	return poSectionObject;
}

void Light::fn_vUpdateSectorSection (void)
{
	POSITION xSectionObjectPos;
	CPA_SectionObject* poSectionObject;

	for (poSectionObject=ms_oListOfSectionObject.GetHeadElement(xSectionObjectPos); poSectionObject; poSectionObject=ms_oListOfSectionObject.GetNextElement(xSectionObjectPos) )
		poSectionObject->fn_vSectionSaved ();
}


// constructor for new lights
Light::Light(CPA_MainWorld *p_oMainWorld, CPA_ObjectDLLBase *p_oDLL, CString csFileName ):
CPA_SaveObject (p_oDLL, C_szLightTypeName, E_ss_Responsible )
{
	char szSectionName[SCR_CV_ui_Cfg_MaxLenName];
	char* pszLevelsDataPath = fn_szGetLevelsDataPath();

	m_lType = fn_lGetTypeLight ();
	m_poNearSuperObject = NULL;
	m_poFarSuperObject = NULL;
//CHINA WFQ (MT) 16/02/98 {
	m_poInterSuperObject = NULL;
	m_poExterSuperObject = NULL;
//ENDCHINA WFQ }
    m_bSelect=FALSE;
	m_oSectorList.RemoveAll ();
	m_fIntensity=1.f;
	m_b3DsMaxLight=FALSE;
	m_bLightOn=TRUE;
//ROMTEAM BlendRLI (Ionut Grozea 24/02/98)
	m_lIsTypePaint = 0;
//ENDROMTEAM BlendRLI (Ionut Grozea)

	// alpha default value
	GEO_tdstColor stColor;
	GLI_xGetLightColor((GLI_tdxHandleToLight)GetEngineStruct(), &stColor);
	stColor.xA = 1.0;
	GLI_xSetLightColor((GLI_tdxHandleToLight)GetEngineStruct(), &stColor);

	// section object
	SetSectionData	( this ) ;
	SetCallBackSave ( fn_vCallbackSave ) ;
	SetDataPath		( pszLevelsDataPath ) ;
	SCR_fn_v_RdL0_ComputeSectionName ( szSectionName, (char *)(LPCSTR)csFileName, pszLightAction, " ");
	SetReferencedSectionName(szSectionName);

	// name
	if    ( fn_eRename ("Light1") != E_mc_None)
		SetDefaultValidName ( ) ;

	fn_vUpdateSectionName ( ) ;

	SetExistingSection (FALSE) ;


	// matrix section
	char szMatrixName [SCR_CV_ui_Cfg_MaxLenName];
	SCR_fn_v_RdL0_ComputeSectionName
	(
		szMatrixName,
		csFileName.GetBuffer (80),
		pszMatrixAction,
		GetName().GetBuffer (80)
	);
	m_poMatrixSection = new CPA_SectionObject ( szMatrixName, pszLevelsDataPath, Light::fn_vCallbackSaveMatrix, this, FALSE );
}


// constructor for lights loads
Light::Light(CPA_MainWorld *p_oMainWorld, CPA_ObjectDLLBase *p_oDLL, GLI_tdxHandleToLight hEngineLight, CString csName, CString csFileName, CString csSectionName ):
CPA_SaveObject (p_oDLL, C_szLightTypeName, E_ss_Responsible ),
CPA_EdMot<GLI_tdxHandleToLight>(hEngineLight)
{
	char szSectionName[SCR_CV_ui_Cfg_MaxLenName];
	char* pszLevelsDataPath = fn_szGetLevelsDataPath();

	m_lType = fn_lGetTypeLight ();
	m_poNearSuperObject = NULL;
	m_poFarSuperObject = NULL;
//CHINA WFQ (MT) 16/02/98 {
	m_poInterSuperObject = NULL;
	m_poExterSuperObject = NULL;
//ENDCHINA WFQ }
    m_bSelect=FALSE;
	m_oSectorList.RemoveAll ();
	m_b3DsMaxLight=FALSE;
	m_bLightOn=TRUE;
//ROMTEAM BlendRLI (Ionut Grozea 24/02/98)
	m_lIsTypePaint = 0;
//ENDROMTEAM BlendRLI (Ionut Grozea)

	// compute intensity
	float fMaxRGBValue;
	GEO_tdxColorValue xRValue, xGValue, xBValue, xAValue;
	fn_vGetColor ( &xRValue, &xGValue, &xBValue, &xAValue );

	fMaxRGBValue = max ( fabs((double)xRValue), fabs((double)xGValue) );
	fMaxRGBValue = max ( fMaxRGBValue, fabs((double)xBValue) );

	// jack's law : if all values less than 1 -> intensity=1
	m_fIntensity = max ( fMaxRGBValue, 1.f );

	// section object
	SetSectionData	( this ) ;
	SetCallBackSave ( fn_vCallbackSave ) ;
	SetDataPath		( pszLevelsDataPath ) ;
	SCR_fn_v_RdL0_ComputeSectionName ( szSectionName, (char *)(LPCSTR)csFileName, (char *)(LPCSTR)csSectionName, " ");
	SetReferencedSectionName(szSectionName);

	// name
	if    ( fn_eRename (csName) != E_mc_None)
		SetDefaultValidName ( ) ;

	fn_vUpdateSectionName ( ) ;

	SetExistingSection (TRUE) ;


	// matrix section
	char szMatrixName [SCR_CV_ui_Cfg_MaxLenName];
	SCR_fn_v_RdL0_ComputeSectionName
	(
		szMatrixName,
		csFileName.GetBuffer (80),
		pszMatrixAction,
		GetName().GetBuffer (80)
	);
	m_poMatrixSection = new CPA_SectionObject ( szMatrixName, pszLevelsDataPath, Light::fn_vCallbackSaveMatrix, this, TRUE );

}


Light::Light( Light& rLight):
CPA_SaveObject (ms_poInterface, C_szLightTypeName, E_ss_Responsible ),
CPA_EdMot<GLI_tdxHandleToLight>()
{
	char szSectionName[SCR_CV_ui_Cfg_MaxLenName];
	char szFileName[SCR_CV_ui_Cfg_MaxLenName];
	char szActionName[SCR_CV_ui_Cfg_MaxLenName];
	char* pszLevelsDataPath = fn_szGetLevelsDataPath();

	fn_vCopyLight ( GetStruct(), rLight.GetStruct() );

	m_lType = rLight.m_lType;
	m_poNearSuperObject = NULL;
	m_poFarSuperObject = NULL;
//CHINA WFQ (MT) 16/02/98 {
	m_poInterSuperObject = NULL;
	m_poExterSuperObject = NULL;
//ENDCHINA WFQ }
    m_bSelect = FALSE;
	m_oSectorList.RemoveAll ();
	m_oSectorList.AddTail ( &(rLight.m_oSectorList) );
	m_fIntensity = rLight.m_fIntensity;
	m_b3DsMaxLight = FALSE;
	m_bLightOn=TRUE;
//ROMTEAM BlendRLI (Ionut Grozea 24/02/98)
	m_lIsTypePaint = 0;
//ENDROMTEAM BlendRLI (Ionut Grozea)

	// section object
	SetSectionData	( this ) ;
	SetCallBackSave ( fn_vCallbackSave ) ;
	SetDataPath		( pszLevelsDataPath ) ;
	SCR_fn_v_RdL0_SplitSectionName ( rLight.GetReferencedSectionName().GetBuffer(SCR_CV_ui_Cfg_MaxLenName), szFileName, szActionName, NULL );
	SCR_fn_v_RdL0_ComputeSectionName ( szSectionName, szFileName, szActionName, " ");

	SetReferencedSectionName(szSectionName);

	// name
	CString csNewName;
	BOOL bValid = FALSE;
	int iIndex = 1;
	
	while (!bValid)
	{
		csNewName.Format("Copy%dOf%s", iIndex, rLight.GetName());
		if (fn_eRename(csNewName) != E_mc_None)
			iIndex++;
		else
			bValid = TRUE;
	}
//		SetDefaultValidName ( ) ;
	fn_vUpdateSectionName ( ) ;
	SetExistingSection (FALSE) ;

	// matrix section
	char szMatrixName [SCR_CV_ui_Cfg_MaxLenName];
	SCR_fn_v_RdL0_ComputeSectionName
	(
		szMatrixName,
		szFileName,
		pszMatrixAction,
		GetName().GetBuffer (80)
	);
	m_poMatrixSection = new CPA_SectionObject ( szMatrixName, pszLevelsDataPath, Light::fn_vCallbackSaveMatrix, this, FALSE );
}


Light::~Light(void)
{
}

//-------------------------------------------------------------------
// Data
//-------------------------------------------------------------------
long Light::GetDataType (void)
{
	return HIE_C_ulEDT_Light;
}

void* Light::GetData (void)
{
	return GetStruct();
}



//-------------------------------------------------------------------
// SAVE
//-------------------------------------------------------------------
void Light::fn_vNotifySave (void)
{
	if (!GetInterface()->fn_bIsExistFile() && !ms_bNotifyHeader)
	{
		SCR_fn_v_SvL1_RegisterNotify ( GetInterface()->fn_csGetSaveFileName().GetBuffer(256), Light::fn_vCallbackHeader, NULL, SCR_EA_Ntfy_AddSection );
		ms_bNotifyHeader=TRUE;
	}

	// for reinsertion
	if (GetCurrentNotification() == SCR_EA_Ntfy_DeleteSection)
	{
		fn_vNotifyRestore();
		return;
	}
	
	fn_vUpDateSectorSection ();

	CPA_SaveObject::fn_vNotifySave ();
	m_poMatrixSection->fn_vWriteSection ();

	// all the sector sections
	POSITION xSectorPos;
	CPA_SectionObject* poSaveObject;

	for ( poSaveObject=m_oListOfSectorSection.GetHeadElement(xSectorPos); poSaveObject; poSaveObject=m_oListOfSectorSection.GetNextElement(xSectorPos) )
			poSaveObject->fn_vNotifySection ( SCR_EA_Ntfy_AddOrRebuildSection );
}

void Light::fn_vNotifyUnSave (void)
{
	fn_vUpDateSectorSection ();


	CPA_SaveObject::fn_vNotifyUnSave ();
	m_poMatrixSection->fn_vDeleteSection ();

	// all the sector sections
	POSITION xSectorPos;
	CPA_SectionObject* poSaveObject;

	for ( poSaveObject=m_oListOfSectorSection.GetHeadElement(xSectorPos); poSaveObject; poSaveObject=m_oListOfSectorSection.GetNextElement(xSectorPos) )
			poSaveObject->fn_vNotifySection ( SCR_EA_Ntfy_AddOrRebuildSection );
}

void Light::fn_vNotifyRestore (void)
{
	fn_vUpDateSectorSection ();

	CPA_SaveObject::fn_vNotifyRestore ();
	m_poMatrixSection->fn_vRestoreSection();

	// all the sector sections
	POSITION xSectorPos;
	CPA_SectionObject* poSaveObject;

	for ( poSaveObject=m_oListOfSectorSection.GetHeadElement(xSectorPos); poSaveObject; poSaveObject=m_oListOfSectorSection.GetNextElement(xSectorPos) )
			poSaveObject->fn_vNotifySection ( SCR_EA_Ntfy_AddOrRebuildSection );
}

void Light::fn_vNotifyRename (void)
{
	CPA_SaveObject::fn_vNotifyRename ();



	// matrix section
	char szFileName [SCR_CV_ui_Cfg_MaxLenName];
	char szActionName [SCR_CV_ui_Cfg_MaxLenName];
	char szName [SCR_CV_ui_Cfg_MaxLenName];
	char szNewMatrixSectionName [SCR_CV_ui_Cfg_MaxLenName];

	CString csOldMatrixSectionName = m_poMatrixSection->GetReferencedSectionName ();
	SCR_fn_v_RdL0_SplitSectionName
	(
		(char*) (LPCSTR) csOldMatrixSectionName,
		szFileName,
		szActionName,
		szName
	);
	SCR_fn_v_RdL0_ComputeSectionName
	(
		szNewMatrixSectionName,
		szFileName,
		szActionName,
		(char*) (LPCSTR) GetName()
	);
	m_poMatrixSection->fn_vRenameSection ( szNewMatrixSectionName );

	// all the sector sections
	POSITION xSectorPos;
	CPA_SectionObject* poSaveObject;

	for ( poSaveObject=m_oListOfSectorSection.GetHeadElement(xSectorPos); poSaveObject; poSaveObject=m_oListOfSectorSection.GetNextElement(xSectorPos) )
		poSaveObject->fn_vNotifySection ( SCR_EA_Ntfy_AddOrRebuildSection );
}

void Light::fn_vUpdateReference (CPA_SaveObject *pReferencedObject)
{
	fn_vNotifySave ();
}



void Light::fn_vUpDateSectorSection (void)
{
	POSITION xSectorPos;
	CPA_SuperObject* poSectorSuperObject;
	CPA_List <CPA_SectionObject> oListRemoveSection;
	POSITION xSectionPos;
	CPA_SectionObject* poSectionObject;

	// for all sectors in the list
	for (poSectorSuperObject=m_oSectorList.GetHeadElement(xSectorPos); poSectorSuperObject; poSectorSuperObject=m_oSectorList.GetNextElement(xSectorPos) )
	{
		for (poSectionObject=m_oListOfSectorSection.GetHeadElement(xSectionPos); poSectionObject && poSectionObject->GetSectionData()!=poSectorSuperObject; poSectionObject=m_oListOfSectorSection.GetNextElement(xSectionPos) );

		// no
		if (!poSectionObject)
		{
			CPA_SectionObject* poNewSectionObject = fn_poFindSectionObject ( poSectorSuperObject );
			m_oListOfSectorSection.AddTail ( poNewSectionObject );
		}
	}

	// for all sections
	for (poSectionObject=m_oListOfSectorSection.GetHeadElement(xSectionPos); poSectionObject; poSectionObject=m_oListOfSectorSection.GetNextElement(xSectionPos) )
	{
		CPA_SuperObject* poSuperSector = (CPA_SuperObject*) poSectionObject->GetSectionData();

		// this section is already referenced ?
		for (poSectorSuperObject=m_oSectorList.GetHeadElement(xSectorPos); poSectorSuperObject && poSectorSuperObject!=poSuperSector; poSectorSuperObject=m_oSectorList.GetNextElement(xSectorPos) );

		// no
		if (!poSectorSuperObject)
		{
			// notify
			poSectionObject->fn_vNotifySection ( SCR_EA_Ntfy_AddOrRebuildSection );
			oListRemoveSection.AddTail (poSectionObject);
		}
	}

	// delete sections
	POSITION xRemoveSectionPos;

	for ( poSectionObject=oListRemoveSection.GetHeadElement(xRemoveSectionPos); poSectionObject; poSectionObject=oListRemoveSection.GetNextElement(xRemoveSectionPos) )
	{
		POSITION xFindSectionPos = m_oListOfSectorSection.Find (poSectionObject);
		m_oListOfSectorSection.RemoveAt (xFindSectionPos);

	}

}


//----- color
void Light::fn_vSetColor ( GEO_tdxColorValue xR, GEO_tdxColorValue xG, GEO_tdxColorValue xB, GEO_tdxColorValue xA )
{
	GEO_tdstColor stColor;
	stColor.xR = xR;
	stColor.xG = xG;
	stColor.xB = xB;
	stColor.xA = xA;

	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_RGB && ((ModifLightRGB*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLightRGB*)poModif)->fn_vSetLightRGB ( stColor );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLightRGB (this, stColor, FALSE));


	GetInterface()->GetInterface()->fn_vUpdateAll (E_mc_JustDraw);
}

void Light::fn_vGetColor ( GEO_tdxColorValue* xR, GEO_tdxColorValue* xG, GEO_tdxColorValue* xB, GEO_tdxColorValue* xA )
{
	GEO_tdstColor stColor;
	GLI_xGetLightColor ( (GLI_tdxHandleToLight)GetEngineStruct(), &stColor );
	*xR = stColor.xR;
	*xG = stColor.xG;
	*xB = stColor.xB;
	*xA = stColor.xA;
}

void Light::fn_vSetLightIntensity ( float& fIntensity )
{
	// if the last modif was a ModifLightAlpha, we erase it
	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_INTENSITY && ((ModifLightIntensity*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLightIntensity*)poModif)->fn_vSetLightIntensity ( fIntensity );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLightIntensity (this, fIntensity, FALSE));
}


void Light::fn_vSetLightAlpha ( MTH_tdxReal& xLittleAlpha, MTH_tdxReal& xBigAlpha )
{
	// if the last modif was a ModifLightAlpha, we erase it
	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_ALPHA && ((ModifLightAlpha*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLightAlpha*)poModif)->fn_vSetLightAlpha ( xLittleAlpha, xBigAlpha );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLightAlpha (this, xLittleAlpha, xBigAlpha, FALSE));
}

void Light::fn_vSetLightNearFar ( MTH_tdxReal& xNear, MTH_tdxReal& xFar )
{
	// if the last modif was a ModifLightAlpha, we erase it
	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_NEARFAR && ((ModifLightNearFar*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLightNearFar*)poModif)->fn_vSetLightNearFar ( xNear, xFar );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLightNearFar (this, xNear, xFar, FALSE));
}

//CHINA WFQ (MT) 16/02/98 {
void Light::fn_vSetLight_Inter_Min_Pos ( MTH3D_tdstVector& xInter_Min_Pos )
{
	// if the last modif was a ModifLightAlpha, we erase it
	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_INTER_MIN && ((ModifLight_Inter_Min_Pos*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLight_Inter_Min_Pos*)poModif)->fn_vSetLight_Inter_Min_Pos ( xInter_Min_Pos );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLight_Inter_Min_Pos (this, xInter_Min_Pos, FALSE));
}

void Light::fn_vSetLight_Inter_Max_Pos ( MTH3D_tdstVector& xInter_Max_Pos )
{
	// if the last modif was a ModifLightAlpha, we erase it
	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_INTER_MAX && ((ModifLight_Inter_Max_Pos*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLight_Inter_Max_Pos*)poModif)->fn_vSetLight_Inter_Max_Pos ( xInter_Max_Pos );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLight_Inter_Max_Pos (this, xInter_Max_Pos, FALSE));
}

void Light::fn_vSetLight_Exter_Min_Pos ( MTH3D_tdstVector& xExter_Min_Pos )
{
	// if the last modif was a ModifLightAlpha, we erase it
	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_EXTER_MIN && ((ModifLight_Exter_Min_Pos*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLight_Exter_Min_Pos*)poModif)->fn_vSetLight_Exter_Min_Pos ( xExter_Min_Pos );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLight_Exter_Min_Pos (this, xExter_Min_Pos, FALSE));
}

void Light::fn_vSetLight_Exter_Max_Pos ( MTH3D_tdstVector& xExter_Max_Pos )
{
	// if the last modif was a ModifLightAlpha, we erase it
	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_EXTER_MAX && ((ModifLight_Exter_Max_Pos*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLight_Exter_Max_Pos*)poModif)->fn_vSetLight_Exter_Max_Pos ( xExter_Max_Pos );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLight_Exter_Max_Pos (this, xExter_Max_Pos, FALSE));
}

void Light::fn_vSetLight_Intensity_Min_Max ( MTH_tdxReal& xIntensity_Min, MTH_tdxReal& xIntensity_Max )
{
	// if the last modif was a ModifLightAlpha, we erase it
	CPA_Modif* poModif;

	if (GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetCount() > 0)
	{
		poModif = GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->ListOfModifs.GetTail();
		if (poModif)
		{
			if (poModif->GetType()==TYPE_CHANGE_INTENSITY_MIN_MAX && ((ModifLight_Intensity_Min_Max*)poModif)->fn_pGetLight()==this)
			{
				if ( poModif->HasBeenDone() )
				{
					((ModifLight_Intensity_Min_Max*)poModif)->fn_vSetLight_Intensity_Min_Max ( xIntensity_Min, xIntensity_Max );
					return;
				}
			}
		}
	}
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLight_Intensity_Min_Max (this, xIntensity_Min, xIntensity_Max, FALSE));
}
//ENDCHINA WFQ }

//--- On/Off


void Light::fn_vSetOnOff (BOOL bState)
{
	GetInterface()->GetInterface()->GetMultiDevice()->GetEditManager()->AskFor (new ModifLightOnOff (this, bState, FALSE));
//ROMTEAM color lights Ionut Grozea 10-08-1998
	if (GetInterface()->fn_bGetColorMode())
		Set2Colors();
//ROMTEAM color lights Ionut Grozea 10-08-1998
	GetInterface()->GetInterface()->fn_vUpdateAll (E_mc_JustDraw);
}


BOOL Light::fn_bGetOnOff (void)
{
	long lState;
//ANNECY CT 19/02/98{
	lState=GLI_ucGetObjectLighted ( (GLI_tdxHandleToLight)GetEngineStruct()) & GLI_C_LightPerso;
//ENDANNECY CT}
	return lState;
}

//ROMTEAM BlendRLI (Ionut Grozea 24/02/98)
BOOL Light::fn_bIsPaint (void)
{
	if (m_lIsTypePaint==1) return 1;
	else
	return GLI_ucGetPaintingLightFlag ((GLI_tdxHandleToLight)GetEngineStruct());
}

BOOL Light::fn_bIsAlphaLight (void)
{
	return (GLI_ucGetAlphaLightFlag ((GLI_tdxHandleToLight)GetEngineStruct()) == 1);
}

BOOL Light::fn_bIsNoAlphaLight (void)
{
	return (GLI_ucGetAlphaLightFlag ((GLI_tdxHandleToLight)GetEngineStruct()) == 2);
}

void Light::fn_vSetTypePaint()
{
	m_lIsTypePaint = 1;
}

void Light::fn_vSetLightForAlpha()
{
	GEO_tdstColor stColor;
	GLI_xGetLightColor ( (GLI_tdxHandleToLight)GetEngineStruct(), &stColor );
	stColor.xA = stColor.xR;
 	fn_vSetColor(stColor.xR , stColor.xG , stColor.xB ,   stColor.xA); 
}

void Light::fn_vSetTypeNormal()
{
	m_lIsTypePaint = 0;
}
//ENDROMTEAM BlendRLI (Ionut Grozea)

void Light::fn_vSetOnDecor (BOOL bState)
{
//ANNECY CT 19/02/98{
	GLI_vSetLightMap( (GLI_tdxHandleToLight)GetEngineStruct(), bState );
//ENDANNECY CT}
	GetInterface()->fn_vComputeRLI ();
}


BOOL Light::fn_bGetOnDecor (void)
{
	unsigned char ucState;
//ANNECY CT 19/02/98{
	ucState = GLI_ucGetObjectLighted( (GLI_tdxHandleToLight)GetEngineStruct() ) & GLI_C_LightMap;
//ENDANNECY CT}
	return ucState;
}


void Light::fn_vCallbackHeader ( SCR_tdst_File_Description *p_stFile,	char *_p_szSectionName, void *_p_vData,SCR_tde_Ntfy_Action	_eAction )
{
	if (_eAction==SCR_EA_Ntfy_AddSection)
	{
		char szBuffer [256];
		char szTime [256];
		char szDate [256];
		SCR_M_SvL0_SaveScriptFileHeader(p_stFile);
		SCR_M_SvL0_SaveComment(p_stFile, "Generated by light editor");
		_strtime(szTime);
		_strdate(szDate);
		sprintf(szBuffer, "Created date : %s %s", szDate, szTime);
		SCR_M_SvL0_SaveComment(p_stFile, szBuffer);
		SCR_M_SvL0_SaveBlankLine(p_stFile);
		SCR_M_SvL0_SaveDirective(p_stFile, "SetCurrentFileDouble",SCR_CC_C_Cfg_NoChar);

		SCR_fn_v_SvL0_SaveParameters_MP(p_stFile, SCR_EF_SvL0_Normal, 2, "1", "1.00");
		SCR_M_SvL0_SaveBlankLine(p_stFile);
	}
}


void Light::fn_vCallbackSave ( SCR_tdst_File_Description *_p_stFile, char *_p_szSectionName, void *_p_vData,SCR_tde_Ntfy_Action	_eAction )
{
	Light* poLight = (Light*)_p_vData;

	switch (_eAction)
	{
	case SCR_EA_Ntfy_AddSection :
		SCR_fn_v_SvL1_ToEndSection (_p_stFile);

	case SCR_EA_Ntfy_ModifySection :
	case SCR_EA_Ntfy_RebuildSection :
		{
			char szObjectName [SCR_CV_ui_Cfg_MaxLenName];
			char szFullLightName [SCR_CV_ui_Cfg_MaxLenName];
//CHINA WFQ (MT) 16/02/98 {
//ANNECY jt {
//ROMTEAM Z_Light Ionut Grozea 01-05-1998
			char* szType [] = { "Invalid", "Parallel", "Spherical", "HotSpot", "Ambient","ParallelBox", "Fog" ,"Z_Light"};
//ENDROMTEAM Z_Light Ionut Grozea 01-05-1998
//ENDANNECY jt }
//ENDCHINA WFQ }
            double afColor [4];
 
			GEO_tdstColor stColor;
			GLI_xGetLightColor ( (GLI_tdxHandleToLight)(poLight->GetStruct()), &stColor );
 			strcpy ( szObjectName, poLight->GetName() );

			SCR_fn_v_RdL0_ComputeSectionName
			(
				szFullLightName,
				NULL,
				pszLightAction,
				szObjectName
			);

			afColor[0] = (double)stColor.xR;
			afColor[1] = (double)stColor.xG;
			afColor[2] = (double)stColor.xB;
			afColor[3] = (double)stColor.xA;
 			SCR_M_SvL0_SaveBeginSection(_p_stFile, szFullLightName, SCR_EF_SvL0_Normal );
			SCR_fn_v_SvL0_SaveParameters_AP ( _p_stFile, SCR_EF_SvL0_Normal, 1, szType + poLight->fn_lGetType() );

			//--- common part
//ROMTEAM BlendRLI (Ionut Grozea 24/02/98)
			if (poLight->fn_bIsPaint())
			{
				SCR_M_SvL0_SaveEntry( _p_stFile, "PaintLight", SCR_CC_C_Cfg_EOL );
			}

			if (poLight->fn_bIsNoAlphaLight())
			{
				SCR_M_SvL0_SaveEntry( _p_stFile, "NoAlphaLight", SCR_CC_C_Cfg_EOL );
			}

//ENDROMTEAM BlendRLI (Ionut Grozea)
//ANNECY CT 19/02/98{
			SCR_M_SvL0_SaveEntry( _p_stFile, "SetState", SCR_CC_C_Cfg_NoChar );
			SCR_fn_v_SvL0_SaveParameters_MP ( _p_stFile, SCR_EF_SvL0_Normal, 1, poLight->m_bLightOn ? "1" : "0");
			
			CString csObjectLighted;
			csObjectLighted.Format("%d",GLI_ucGetObjectLighted((GLI_tdxHandleToLight)poLight->GetEngineStruct() ) );
			SCR_M_SvL0_SaveEntry( _p_stFile, "ObjectLighted", SCR_CC_C_Cfg_NoChar );
			SCR_fn_v_SvL0_SaveParameters_MP ( _p_stFile, SCR_EF_SvL0_Normal, 1,csObjectLighted); 
//ENDANNECY CT}

			SCR_M_SvL0_SaveEntry( _p_stFile, "SetColor", SCR_CC_C_Cfg_NoChar );

			SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 4, afColor );

 			//--- specific part
			switch (poLight->fn_lGetType())
			{
				case GLI_C_lSphericalLight:
				{
					double adNearFar [2];
					MTH_tdxReal xNear, xFar;
					GLI_vGetLightNearFar ( (GLI_tdxHandleToLight) (poLight->GetStruct()), &xNear, &xFar );

					adNearFar [0] = xNear;
					adNearFar [1] = xFar;

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetNearFar", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 2, adNearFar );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetMatrix", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_Normal, 1, (char *)(LPCSTR) poLight->GetMatrixSection()->GetReferencedSectionName() );

				}
				break;

				case GLI_C_lHotSpotLight:
				{
					MTH_tdxReal afNearFar [2];
					MTH_tdxReal afAlpha [2];
					double adNearFar [2];
					double adAlpha [2];

					GLI_vGetLightNearFar ( (GLI_tdxHandleToLight) (poLight->GetStruct()), afNearFar, afNearFar+1 );
					GLI_vGetLightAlphas ( (GLI_tdxHandleToLight) (poLight->GetStruct()), afAlpha, afAlpha+1 );

					adNearFar[0] = afNearFar[0];
					adNearFar[1] = afNearFar[1];
					adAlpha[0] = afAlpha[0];
					adAlpha[1] = afAlpha[1];

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetNearFar", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 2, adNearFar );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetAlphas", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 2, adAlpha );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetMatrix", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_Normal, 1, (char *)(LPCSTR) poLight->GetMatrixSection()->GetReferencedSectionName() );

				}
				break;

				case GLI_C_lParallelLight :
				{
					SCR_M_SvL0_SaveEntry( _p_stFile, "SetMatrix", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_Normal, 1, (char *)(LPCSTR) poLight->GetMatrixSection()->GetReferencedSectionName() );
				}
				break;

				case GLI_C_lAmbientLight :
				{
					SCR_M_SvL0_SaveEntry( _p_stFile, "SetMatrix", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_Normal, 1, (char *)(LPCSTR) poLight->GetMatrixSection()->GetReferencedSectionName() );
				}
				break;
//ROMTEAM Z_Light Ionut Grozea 01-05-1998
/*	
			case GLI_C_lZ_Light:
				{
					double adNearFar [2];
					MTH_tdxReal xNear, xFar;
					GLI_vGetLightNearFar ( (GLI_tdxHandleToLight) (poLight->GetStruct()), &xNear, &xFar );

					adNearFar [0] = xNear;
					adNearFar [1] = xFar;

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetNearFar", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 2, adNearFar );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetMatrix", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_Normal, 1, (char *)(LPCSTR) poLight->GetMatrixSection()->GetReferencedSectionName() );

				}
				break;
*/
//ENDROMTEAM Z_Light Ionut Grozea 01-05-1998
//CHINA WFQ (MT) 16/02/98 {
				case GLI_C_lParallelLight_With_Limit :
				{
					MTH3D_tdstVector xInterMinPos, xInterMaxPos, xExterMinPos, xExterMaxPos;
					MTH_tdxReal afIntensity[2];

					double adInterMin [3];
					double adInterMax [3];
					double adExterMin [3];
					double adExterMax [3];
					double adIntensity[2];

					GLI_vGetLight_Inter_Min_Pos ( (GLI_tdxHandleToLight) (poLight->GetStruct()), &xInterMinPos );
					GLI_vGetLight_Inter_Max_Pos ( (GLI_tdxHandleToLight) (poLight->GetStruct()), &xInterMaxPos );
					GLI_vGetLight_Exter_Min_Pos ( (GLI_tdxHandleToLight) (poLight->GetStruct()), &xExterMinPos );
					GLI_vGetLight_Exter_Max_Pos ( (GLI_tdxHandleToLight) (poLight->GetStruct()), &xExterMaxPos );
					GLI_vGetLight_Intensity_Min_Max ( (GLI_tdxHandleToLight) (poLight->GetStruct()), afIntensity, afIntensity+1 );

					adInterMin[0] = xInterMinPos.xX; adInterMin[1] = xInterMinPos.xY; adInterMin[2] = xInterMinPos.xZ;
					adInterMax[0] = xInterMaxPos.xX; adInterMax[1] = xInterMaxPos.xY; adInterMax[2] = xInterMaxPos.xZ;
					adExterMin[0] = xExterMinPos.xX; adExterMin[1] = xExterMinPos.xY; adExterMin[2] = xExterMinPos.xZ;
					adExterMax[0] = xExterMaxPos.xX; adExterMax[1] = xExterMaxPos.xY; adExterMax[2] = xExterMaxPos.xZ;
					adIntensity[0] = afIntensity[0]; adIntensity[1] = afIntensity[1];

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetInterMinPos", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 3, adInterMin );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetInterMaxPos", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 3, adInterMax );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetExterMinPos", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 3, adExterMin );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetExterMaxPos", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 3, adExterMax );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetIntensityMinMax", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 2, adIntensity );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetMatrix", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_Normal, 1, (char *)(LPCSTR) poLight->GetMatrixSection()->GetReferencedSectionName() );

				}
				break;
//ENDCHINA WFQ }

//ANNECY jt {
				case GLI_C_lFogLight :
				{
					MTH_tdxReal afNearFar [2];
					MTH_tdxReal afAlpha [2];
					MTH_tdxReal afIntensity[2];

					double adNearFar [2];
					double adAlpha [2];
					double adIntensity[2];

					GLI_vGetLightNearFar ( (GLI_tdxHandleToLight) (poLight->GetStruct()), afNearFar, afNearFar+1 );
					GLI_vGetLightAlphas ( (GLI_tdxHandleToLight) (poLight->GetStruct()), afAlpha, afAlpha+1 );
					GLI_vGetLight_Intensity_Min_Max ( (GLI_tdxHandleToLight) (poLight->GetStruct()), afIntensity, afIntensity+1 );


					adNearFar[0] = afNearFar[0];
					adNearFar[1] = afNearFar[1];
					adAlpha[0] = afAlpha[0];
					adAlpha[1] = afAlpha[1];
					adIntensity[0] = afIntensity[0];
					adIntensity[1] = afIntensity[1];

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetNearFar", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 2, adNearFar );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetAlphas", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 2, adAlpha );
	
					SCR_M_SvL0_SaveEntry( _p_stFile, "SetIntensityMinMax", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 2, adIntensity );

					SCR_M_SvL0_SaveEntry( _p_stFile, "SetMatrix", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_Normal, 1, (char *)(LPCSTR) poLight->GetMatrixSection()->GetReferencedSectionName() );

					SCR_M_SvL0_SaveEntry( _p_stFile, "BackgroundColor", SCR_CC_C_Cfg_NoChar );
					GLI_xGetLightBackgroundColor ( (GLI_tdxHandleToLight)(poLight->GetStruct()), &stColor );
					afColor[0] = (double)stColor.xR;
					afColor[1] = (double)stColor.xG;
					afColor[2] = (double)stColor.xB;
					afColor[3] = (double)stColor.xA;
					SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayDouble, 2, 4, afColor );
				}
				break;
//ENDANNECY jt }

			}

			SCR_M_SvL0_SaveEndSection(_p_stFile, SCR_CC_C_Cfg_EOL);
			poLight->fn_vSectionSaved ();

		} break;

		case SCR_EA_Ntfy_DeleteSection :
			poLight->fn_vSectionDeleted ();
			break;

	}
}



void Light::fn_vCallbackSaveMatrix ( SCR_tdst_File_Description *_p_stFile,	char *_p_szSectionName, void *_p_vData,SCR_tde_Ntfy_Action	_eAction )
{
	MTH3D_tdstVector stTranslation, stSI, stSJ, stSK, stRI, stRJ, stRK;;

	char szMatrixName [SCR_CV_ui_Cfg_MaxLenName];
	char szMatrixSectionName [SCR_CV_ui_Cfg_MaxLenName];

	switch (_eAction)
	{
	case SCR_EA_Ntfy_DeleteSection: return;

	case SCR_EA_Ntfy_AddSection :
			SCR_fn_v_SvL1_ToEndSection (_p_stFile);

	case SCR_EA_Ntfy_ModifySection :
	case SCR_EA_Ntfy_RebuildSection :

		Light* poLight = (Light*)_p_vData;
		GEO_tdxHandleToMatrix hMatrix = HIE_fn_hGetSuperObjectMatrix( (HIE_tdxHandleToSuperObject)(poLight->GetSuperObject())->GetStruct() );

		strcpy ( szMatrixName, poLight->GetName() );

		SCR_fn_v_RdL0_ComputeSectionName
		(
			szMatrixSectionName,
			NULL,
			pszMatrixAction,
			szMatrixName
		);

		POS_fn_vGetTranslationVector( hMatrix, &stTranslation );
		POS_fn_vGetScaleMatrix( hMatrix, &stSI, &stSJ, &stSK );
		POS_fn_vGetRotationMatrix( hMatrix, &stRI, &stRJ, &stRK );

		// array
		float		a3_MatrixTranslation	[3];
		float		a9_MatrixScale			[9];
		float		a9_MatrixRotation		[9];

		// translation
		a3_MatrixTranslation [0] = (float)MTH3D_M_xGetXofVector(&stTranslation);
		a3_MatrixTranslation [1] = (float)MTH3D_M_xGetYofVector(&stTranslation);
		a3_MatrixTranslation [2] = (float)MTH3D_M_xGetZofVector(&stTranslation);

		// scale
		a9_MatrixScale [0] = (float)(MTH3D_M_xGetXofVector(&stSI));
		a9_MatrixScale [1] = (float)(MTH3D_M_xGetYofVector(&stSI));
		a9_MatrixScale [2] = (float)(MTH3D_M_xGetZofVector(&stSI));
		a9_MatrixScale [3] = (float)(MTH3D_M_xGetXofVector(&stSJ));
		a9_MatrixScale [4] = (float)(MTH3D_M_xGetYofVector(&stSJ));
		a9_MatrixScale [5] = (float)(MTH3D_M_xGetZofVector(&stSJ));
		a9_MatrixScale [6] = (float)(MTH3D_M_xGetXofVector(&stSK));
		a9_MatrixScale [7] = (float)(MTH3D_M_xGetYofVector(&stSK));
		a9_MatrixScale [8] = (float)(MTH3D_M_xGetZofVector(&stSK));

		// rotation
		a9_MatrixRotation [0] = (float)(MTH3D_M_xGetXofVector(&stRI));
		a9_MatrixRotation [1] = (float)(MTH3D_M_xGetYofVector(&stRI));
		a9_MatrixRotation [2] = (float)(MTH3D_M_xGetZofVector(&stRI));
		a9_MatrixRotation [3] = (float)(MTH3D_M_xGetXofVector(&stRJ));
		a9_MatrixRotation [4] = (float)(MTH3D_M_xGetYofVector(&stRJ));
		a9_MatrixRotation [5] = (float)(MTH3D_M_xGetZofVector(&stRJ));
		a9_MatrixRotation [6] = (float)(MTH3D_M_xGetXofVector(&stRK));
		a9_MatrixRotation [7] = (float)(MTH3D_M_xGetYofVector(&stRK));
		a9_MatrixRotation [8] = (float)(MTH3D_M_xGetZofVector(&stRK));

		SCR_M_SvL0_SaveBeginSection(_p_stFile, szMatrixSectionName, SCR_CC_C_Cfg_EOL );

		SCR_M_SvL0_SaveEntry( _p_stFile, "MatrixTranslation", SCR_CC_C_Cfg_NoChar );
			SCR_fn_v_SvL0_SaveParameters_MP( _p_stFile, SCR_EF_SvL0_ArrayFloat, 2, 3, a3_MatrixTranslation );

		SCR_M_SvL0_SaveEntry( _p_stFile, "MatrixScale", SCR_CC_C_Cfg_NoChar );
			SCR_fn_v_SvL0_SaveParameters_MP ( _p_stFile, SCR_EF_SvL0_ArrayFloat, 2, 9, a9_MatrixScale );

		SCR_M_SvL0_SaveEntry( _p_stFile, "MatrixRotation", SCR_CC_C_Cfg_NoChar );
			SCR_fn_v_SvL0_SaveParameters_MP ( _p_stFile, SCR_EF_SvL0_ArrayFloat, 2, 9, a9_MatrixRotation );

		SCR_M_SvL0_SaveEndSection(_p_stFile, SCR_CC_C_Cfg_EOL);
	}
}



// this function save the section into the sector script
void Light::fn_vCallBackSaveSector ( SCR_tdst_File_Description *_p_stFile,	char *_p_szSectionName, void *_p_vData,SCR_tde_Ntfy_Action	_eAction )
{
	CPA_SuperObject* poSector = (CPA_SuperObject*) _p_vData;

	switch (_eAction)
	{
	case SCR_EA_Ntfy_AddSection :
	case SCR_EA_Ntfy_ModifySection :
	case SCR_EA_Ntfy_RebuildSection :
		{
			char szLigthSubSectionName[SCR_CV_ui_Cfg_MaxLenName];
			POSITION xSuperObjectPos;
			CPA_SuperObject* poSuperObject;

			Light* poLight;
			CPA_List<CPA_SuperObject>* poList = ms_poInterface->GetInterface()->GetObjectListByType	(C_szLightTypeName);

			sprintf ( szLigthSubSectionName, pszLightSectorSubSection );
			strcat ( szLigthSubSectionName, ":" );
			SCR_g_ui_SvL0_IndentationLevel=1;
			SCR_M_SvL0_SaveBeginSection (_p_stFile, szLigthSubSectionName, SCR_CC_C_Cfg_EOL );

			for ( poSuperObject=poList->GetHeadElement(xSuperObjectPos); xSuperObjectPos; poSuperObject=poList->GetNextElement(xSuperObjectPos) )
			{
				CString csLightSectionName;

				poLight = (Light*) poSuperObject->GetObject();
				if (poLight->fn_pGetSectorList()->Find(poSector))
				{
					csLightSectionName=poLight->GetReferencedSectionName ();

					SCR_M_SvL0_SaveEntry( _p_stFile, "AddLstStaticLights", SCR_CC_C_Cfg_NoChar );
						SCR_fn_v_SvL0_SaveParameters_MP ( _p_stFile, SCR_EF_SvL0_Normal, 1, csLightSectionName );
				}

			}

			SCR_M_SvL0_SaveEndSection(_p_stFile, SCR_CC_C_Cfg_EOL);
			SCR_g_ui_SvL0_IndentationLevel=0;


		} break;

	}
}


void Light::fn_vSetEngineMatrix (BOOL bDraw)
{
	GLI_tdxHandleToLight hLight = (GLI_tdxHandleToLight)(GetEngineStruct ());
	GetInterface()->fn_vComputeAbsoluteMatrix (GetSuperObject());
	GEO_tdxHandleToMatrix hGlobalMatrix = HIE_fn_hGetSuperObjectMatrix ( GetSuperObject()->GetStruct() );
	GLI_xSetLightMatrix ( hLight, hGlobalMatrix ) ;
	if (bDraw) GetInterface()->GetInterface()->fn_vUpdateAll (E_mc_JustDraw);
}


void Light::fn_vSelect (void)
{
	if (m_bSelect) return;
	m_bSelect=TRUE;

	if (GetSuperObject()->GetModelName()==csSphericalModelName)
	{
		MTH_tdxReal xNear, xFar;
		GLI_vGetLightNearFar ( (GLI_tdxHandleToLight)GetStruct(), &xNear, &xFar );

		//----- sphere near
		if (!m_poNearSuperObject)
		{
			m_poNearSuperObject = GetInterface()->fn_pGetNewSphereGraphicObject ();
			m_poNearSuperObject->SetEditProtected (TRUE);
			m_poNearSuperObject->SetSuperObjectOwner (GetSuperObject());
		}
		((Geometry3D*)(m_poNearSuperObject->GetObject()))->fn_eSetRadius(&xNear, 0, 0);
		GetInterface()->fn_vComputeInsertMatrix ( m_poNearSuperObject, GetSuperObject() );
		GetInterface()->GetInterface()->fn_bInsertObjectInHierarchy ( m_poNearSuperObject, GetSuperObject(), FALSE );

		//----- far near
		if (!m_poFarSuperObject)
		{
			m_poFarSuperObject = GetInterface()->fn_pGetNewSphereGraphicObject ();
			m_poFarSuperObject->SetEditProtected (TRUE);
			m_poFarSuperObject->SetSuperObjectOwner (GetSuperObject());
		}
		((Geometry3D*)(m_poFarSuperObject->GetObject()))->fn_eSetRadius(&xFar, 0, 0);
		GetInterface()->fn_vComputeInsertMatrix ( m_poFarSuperObject, GetSuperObject() );
		GetInterface()->GetInterface()->fn_bInsertObjectInHierarchy (m_poFarSuperObject, GetSuperObject(), FALSE);
	}

	else if (GetSuperObject()->GetModelName()==csSpotModelName)
	{
		MTH_tdxReal xNear, xFar;
		MTH_tdxReal xLittleAlpha, xBigAlpha;
		MTH_tdxReal xR;
		ACP_tdxHandleOfObject hMot;
		GLI_vGetLightNearFar ( (GLI_tdxHandleToLight)GetStruct(), &xNear, &xFar );
		GLI_vGetLightAlphas ( (GLI_tdxHandleToLight)GetStruct(), &xLittleAlpha, &xBigAlpha);
		MTH3D_tdstVector a2_stPoint [2];


		//----- near cone
		if (!m_poNearSuperObject)
		{
			m_poNearSuperObject = GetInterface()->fn_pGetNewConeGraphicObject ();
			m_poNearSuperObject->SetEditProtected (TRUE);
			m_poNearSuperObject->SetSuperObjectOwner (GetSuperObject());
		}
		xR = (MTH_tdxReal)(xNear * tan (xLittleAlpha/2.));
		((Geometry3D*)(m_poNearSuperObject->GetObject()))->fn_eSetConeBaseRadius( 0, 0, xR);

		hMot = (ACP_tdxHandleOfObject) HIE_fn_hGetSuperObjectObject(m_poNearSuperObject->GetStruct());
		MTH3D_M_vSetVectorElements (a2_stPoint, 0.0, 0.0, 0.0 );
		MTH3D_M_vSetVectorElements (a2_stPoint+1, -xNear, 0.0, 0.0 );
		GEO_vSetListOfPointsOfObject ( hMot, a2_stPoint, 2, 0 );

		GetInterface()->fn_vComputeInsertMatrix ( m_poNearSuperObject, GetSuperObject() );
		GetInterface()->GetInterface()->fn_bInsertObjectInHierarchy (m_poNearSuperObject, GetSuperObject(), FALSE);


		//----- far cone
		if (!m_poFarSuperObject)
		{
			m_poFarSuperObject = GetInterface()->fn_pGetNewConeGraphicObject ();
			m_poFarSuperObject->SetEditProtected (TRUE);
			m_poFarSuperObject->SetSuperObjectOwner (GetSuperObject());
		}
		xR = (MTH_tdxReal)(xFar * tan (xBigAlpha/2.));
		((Geometry3D*)(m_poFarSuperObject->GetObject()))->fn_eSetConeBaseRadius( 0, 0, xR);

		hMot = (ACP_tdxHandleOfObject) HIE_fn_hGetSuperObjectObject(m_poFarSuperObject->GetStruct());
		MTH3D_M_vSetVectorElements (a2_stPoint, 0., 0.0, 0.0 );
		MTH3D_M_vSetVectorElements (a2_stPoint+1, -xFar, 0.0, 0.0 );
		GEO_vSetListOfPointsOfObject ( hMot, a2_stPoint, 2, 0 );

		GetInterface()->fn_vComputeInsertMatrix ( m_poFarSuperObject, GetSuperObject() );
		GetInterface()->GetInterface()->fn_bInsertObjectInHierarchy (m_poFarSuperObject, GetSuperObject(), FALSE);
	}
//ROMTEAM Z_Light Ionut Grozea 01-05-1998
	if (GetSuperObject()->GetModelName()==csZModelName)
	{
		MTH_tdxReal xNear, xFar;
		GLI_vGetLightNearFar ( (GLI_tdxHandleToLight)GetStruct(), &xNear, &xFar );

		//----- Z_Light near
		if (!m_poNearSuperObject)
		{
			m_poNearSuperObject = GetInterface()->fn_pGetNewSphereGraphicObject ();
			m_poNearSuperObject->SetEditProtected (TRUE);
			m_poNearSuperObject->SetSuperObjectOwner (GetSuperObject());
		}
		((Geometry3D*)(m_poNearSuperObject->GetObject()))->fn_eSetRadius(&xNear, 0, 0);
		GetInterface()->fn_vComputeInsertMatrix ( m_poNearSuperObject, GetSuperObject() );
		GetInterface()->GetInterface()->fn_bInsertObjectInHierarchy ( m_poNearSuperObject, GetSuperObject(), FALSE );

		//----- far near
		if (!m_poFarSuperObject)
		{
			m_poFarSuperObject = GetInterface()->fn_pGetNewSphereGraphicObject ();
			m_poFarSuperObject->SetEditProtected (TRUE);
			m_poFarSuperObject->SetSuperObjectOwner (GetSuperObject());
		}
		((Geometry3D*)(m_poFarSuperObject->GetObject()))->fn_eSetRadius(&xFar, 0, 0);
		GetInterface()->fn_vComputeInsertMatrix ( m_poFarSuperObject, GetSuperObject() );
		GetInterface()->GetInterface()->fn_bInsertObjectInHierarchy (m_poFarSuperObject, GetSuperObject(), FALSE);
	}
//ENDROMTEAM Z_Light Ionut Grozea 01-05-1998
//CHINA WFQ (MT) 16/02/98 {
    else if (GetSuperObject()->GetModelName()==csParallelBoxModelName)
	{
		MTH3D_tdstVector xInter_Min_Pos, xInter_Max_Pos, xExter_Min_Pos, xExter_Max_Pos, a2_stPoint[2];
		ACP_tdxHandleOfObject hMot;
		GLI_tdxHandleToLight xGetLight = (GLI_tdxHandleToLight)GetStruct();

		GLI_vGetLight_Inter_Min_Pos ( xGetLight , &xInter_Min_Pos );
		GLI_vGetLight_Inter_Max_Pos ( xGetLight , &xInter_Max_Pos );
		GLI_vGetLight_Exter_Min_Pos ( xGetLight , &xExter_Min_Pos );
		GLI_vGetLight_Exter_Max_Pos ( xGetLight , &xExter_Max_Pos );


		//----- InterBox
		if (!m_poInterSuperObject)
		{
			m_poInterSuperObject = GetInterface()->fn_pGetNewBoxGraphicObject ();
			m_poInterSuperObject->SetEditProtected (TRUE);
			m_poInterSuperObject->SetSuperObjectOwner (GetSuperObject());
		}

		hMot = (ACP_tdxHandleOfObject) HIE_fn_hGetSuperObjectObject(m_poInterSuperObject->GetStruct());
		MTH3D_M_vSetVectorElements (a2_stPoint, xInter_Min_Pos.xX, xInter_Min_Pos.xY, xInter_Min_Pos.xZ );
		MTH3D_M_vSetVectorElements (a2_stPoint+1, xInter_Max_Pos.xX, xInter_Max_Pos.xY, xInter_Max_Pos.xZ );
		GEO_vSetListOfPointsOfObject ( hMot, a2_stPoint, 2, 0 );

		GetInterface()->fn_vComputeInsertMatrix ( m_poInterSuperObject, GetSuperObject() );
		GetInterface()->GetInterface()->fn_bInsertObjectInHierarchy (m_poInterSuperObject, GetSuperObject(), FALSE);

		//----- ExterBox
		if (!m_poExterSuperObject)
		{
			m_poExterSuperObject = GetInterface()->fn_pGetNewBoxGraphicObject ();
			m_poExterSuperObject->SetEditProtected (TRUE);
			m_poExterSuperObject->SetSuperObjectOwner (GetSuperObject());
		}

		hMot = (ACP_tdxHandleOfObject) HIE_fn_hGetSuperObjectObject(m_poExterSuperObject->GetStruct());
		MTH3D_M_vSetVectorElements (a2_stPoint, xExter_Min_Pos.xX, xExter_Min_Pos.xY, xExter_Min_Pos.xZ );
		MTH3D_M_vSetVectorElements (a2_stPoint+1, xExter_Max_Pos.xX, xExter_Max_Pos.xY, xExter_Max_Pos.xZ );
		GEO_vSetListOfPointsOfObject ( hMot, a2_stPoint, 2, 0 );

		GetInterface()->fn_vComputeInsertMatrix ( m_poExterSuperObject, GetSuperObject() );
		GetInterface()->GetInterface()->fn_bInsertObjectInHierarchy (m_poExterSuperObject, GetSuperObject(), FALSE);
	}
//ENDCHINA WFQ }
}

void Light::fn_vUnSelect (BOOL bDraw)
{
	if (!m_bSelect) return;

	//--- remove Near / Far of selected object
	if (m_poNearSuperObject) 
		GetInterface()->GetInterface()->fn_bDeleteObjectInHierarchy (m_poNearSuperObject,FALSE,FALSE,FALSE);
	if (m_poFarSuperObject) 
		GetInterface()->GetInterface()->fn_bDeleteObjectInHierarchy (m_poFarSuperObject,FALSE,FALSE,FALSE);
//	m_poNearSuperObject = NULL;
//	m_poFarSuperObject = NULL;
//CHINA WFQ (MT) 16/02/98 {
	if (m_poInterSuperObject) 
		GetInterface()->GetInterface()->fn_bDeleteObjectInHierarchy (m_poInterSuperObject,FALSE,FALSE,FALSE);
	if (m_poExterSuperObject) 
		GetInterface()->GetInterface()->fn_bDeleteObjectInHierarchy (m_poExterSuperObject,FALSE,FALSE,FALSE);
//	m_poInterSuperObject = NULL;
//	m_poExterSuperObject = NULL;
//ENDCHINA WFQ }
	if (bDraw) 
		GetInterface()->GetInterface()->fn_vUpdateAll (E_mc_JustDraw);
	m_bSelect=FALSE;
}

void Light::fn_vDrawEditObject (void)
{
	MTH_tdxReal xNear, xFar;
	MTH_tdxReal xLittleAlpha, xBigAlpha;

	GLI_vGetLightNearFar ( (GLI_tdxHandleToLight)GetStruct(), &xNear, &xFar );
	GLI_vGetLightAlphas ( (GLI_tdxHandleToLight)GetStruct(), &xLittleAlpha, &xBigAlpha);

	if (GetSuperObject()->GetModelName()==csSphericalModelName)
	{
		if (m_poNearSuperObject) ((Geometry3D*)(m_poNearSuperObject->GetObject()))->fn_eSetRadius(&xNear, 0, 0);
		if (m_poFarSuperObject) ((Geometry3D*)(m_poFarSuperObject->GetObject()))->fn_eSetRadius(&xFar, 0, 0);
	}
	else if (GetSuperObject()->GetModelName()==csSpotModelName)
	{

		MTH_tdxReal xR;
		MTH3D_tdstVector a2_stPoint [2];
		ACP_tdxHandleOfObject hMot;

		if (m_poFarSuperObject)
		{
			xR = (MTH_tdxReal)(xFar * tan (xBigAlpha/2.));
			((Geometry3D*)(m_poFarSuperObject->GetObject()))->fn_eSetConeBaseRadius( 0, 0, xR);
			hMot = (ACP_tdxHandleOfObject) HIE_fn_hGetSuperObjectObject(m_poFarSuperObject->GetStruct());
			MTH3D_M_vSetVectorElements (a2_stPoint, 0.0, 0.0, 0.0 );
			MTH3D_M_vSetVectorElements (a2_stPoint+1, -xFar, 0.0, 0.0);
			GEO_vSetListOfPointsOfObject ( hMot, a2_stPoint, 2, 0 );
		}

		if (m_poNearSuperObject)
		{
			xR = (MTH_tdxReal)(xNear * tan (xLittleAlpha/2.));
			((Geometry3D*)(m_poNearSuperObject->GetObject()))->fn_eSetConeBaseRadius( 0, 0, xR);
			hMot = (ACP_tdxHandleOfObject) HIE_fn_hGetSuperObjectObject(m_poNearSuperObject->GetStruct());
			MTH3D_M_vSetVectorElements (a2_stPoint, 0.0, 0.0, 0.0 );
			MTH3D_M_vSetVectorElements (a2_stPoint+1, -xNear, 0.0, 0.0 );
			GEO_vSetListOfPointsOfObject ( hMot, a2_stPoint, 2, 0 );
		}
	}
//ROMTEAM Z_Light Ionut Grozea 01-05-1998
	if (GetSuperObject()->GetModelName()==csZModelName)
	{
		if (m_poNearSuperObject) ((Geometry3D*)(m_poNearSuperObject->GetObject()))->fn_eSetRadius(&xNear, 0, 0);
		if (m_poFarSuperObject) ((Geometry3D*)(m_poFarSuperObject->GetObject()))->fn_eSetRadius(&xFar, 0, 0);
	}
//ENDROMTEAM Z_Light Ionut Grozea 01-05-1998
//CHINA WFQ (MT) 16/02/98 {
    else if( GetSuperObject()->GetModelName() == csParallelBoxModelName)
	{
		MTH3D_tdstVector xInter_Min_Pos, xInter_Max_Pos, xExter_Min_Pos, xExter_Max_Pos;
		GLI_tdxHandleToLight xGetLight = (GLI_tdxHandleToLight)GetStruct();
		MTH3D_tdstVector a2_stPoint[2];
		ACP_tdxHandleOfObject hMot;

		GLI_vGetLight_Inter_Min_Pos ( xGetLight , &xInter_Min_Pos );
		GLI_vGetLight_Inter_Max_Pos ( xGetLight , &xInter_Max_Pos );
		GLI_vGetLight_Exter_Min_Pos ( xGetLight , &xExter_Min_Pos );
		GLI_vGetLight_Exter_Max_Pos ( xGetLight , &xExter_Max_Pos );
		if ( m_poInterSuperObject )
		{
			hMot = (ACP_tdxHandleOfObject) HIE_fn_hGetSuperObjectObject(m_poInterSuperObject->GetStruct());
			MTH3D_M_vSetVectorElements (a2_stPoint, xInter_Min_Pos.xX, xInter_Min_Pos.xY, xInter_Min_Pos.xZ );
			MTH3D_M_vSetVectorElements (a2_stPoint+1, xInter_Max_Pos.xX, xInter_Max_Pos.xY, xInter_Max_Pos.xZ );

			GEO_vSetListOfPointsOfObject ( hMot, a2_stPoint, 2, 0 );
		}

		if ( m_poExterSuperObject )
		{
			hMot = (ACP_tdxHandleOfObject) HIE_fn_hGetSuperObjectObject(m_poExterSuperObject->GetStruct());
			MTH3D_M_vSetVectorElements (a2_stPoint, xExter_Min_Pos.xX, xExter_Min_Pos.xY, xExter_Min_Pos.xZ );
			MTH3D_M_vSetVectorElements (a2_stPoint+1, xExter_Max_Pos.xX, xExter_Max_Pos.xY, xExter_Max_Pos.xZ );

			GEO_vSetListOfPointsOfObject ( hMot, a2_stPoint, 2, 0 );
		}

    }
//ENDCHINA WFQ }
}


// Change the orientation of the light
void Light::fn_vOrientLight ( MTH3D_tdstVector* pstVertex )
{
	MTH3D_tdstVector n, u, v;
	MTH3D_tdstVector stOrigin;
	BOOL bSpot = GetSuperObject()->GetModelName()==csSpotModelName;

	GEO_tdxHandleToMatrix hAbsoluteLineMatrix = HIE_fn_hGetSuperObjectGlobalMatrix (GetSuperObject()->GetStruct());
	POS_fn_vGetTranslationVector( hAbsoluteLineMatrix, &stOrigin );

	if (bSpot)
	{
		u = stOrigin;
		MTH3D_M_vSubVector(&u,&u,pstVertex);
	}
	else
	{
		u = *pstVertex;
		MTH3D_M_vSubVector(&u,&u,&stOrigin);
	}


	MTH3D_M_vNormalizeVector(&u,&u);

	if (u.xX||u.xY) 
	{
		MTH3D_M_vSetVectorElements ((&v),(-u.xY),(u.xX),0.0); 
	}
	else 
	{ 
		MTH3D_M_vSetVectorElements((&v),0.,(-u.xZ),u.xY); 
	}
	MTH3D_M_vNormalizeVector(&v,&v);
	MTH3D_M_vCrossProductVector((&n),(&u),(&v));

	if (GetSuperObject()->GetModelName()==csParallelModelName)
	{
		v.xX = u.xY;
		v.xY = u.xZ;
		v.xZ = u.xX;
		MTH3D_M_vNormalizeVector(&v,&v);
		n.xX = u.xZ;
		n.xY = u.xX;
		n.xZ = u.xY;
		MTH3D_M_vNormalizeVector(&n,&n);
	}

	POS_fn_vSetIdentityMatrix( hAbsoluteLineMatrix );
	POS_fn_vSetTranslationVector( hAbsoluteLineMatrix, &stOrigin );
	if (bSpot) POS_fn_vSetRotationMatrix( hAbsoluteLineMatrix , &u, &v, &n );
	else POS_fn_vSetRotationMatrix( hAbsoluteLineMatrix , &v, &n, &u );
	GetInterface()->fn_vComputeNewRelativeMatrix (GetSuperObject()->GetStruct());
	fn_vSetEngineMatrix (TRUE);
	fn_vNotifySave ();
}

// color all the sectors affected
void Light::fn_vPaintSector (BOOL bColor)
{
	// First no color for everybody
	CPA_SuperObject* poSector;
	POSITION xBaseObjectPos, xPos;
	CPA_SuperObject* poSuperObject;
	CPA_List<CPA_SuperObject> * poList = GetInterface()->GetInterface()->GetObjectListByType (C_szSectorTypeName);

	// set all the sectors in the list
	for ( poSuperObject=poList->GetHeadElement(xBaseObjectPos); xBaseObjectPos; poSuperObject=poList->GetNextElement(xBaseObjectPos) )
	{
		fn_vColorSector ( poSuperObject, E_lc_NoColor );
	}

	if (bColor)
	{
		for ( poSector = m_oSectorList.GetHeadElement(xPos); poSector; poSector = m_oSectorList.GetNextElement(xPos))
			fn_vColorSector ( poSector, E_lc_Blue );
	}
	GetInterface()->GetInterface()->fn_vUpdateAll (E_mc_JustDraw);

}

void Light::fn_vColorSector ( CPA_SuperObject* poSuperSector, tdeLocalColor eColor   )
{
	POSITION xPos;
	CPA_SuperObjectBase* poChild;

	for ( poChild=poSuperSector->GetHeadElement(xPos); poChild; poChild=poSuperSector->GetNextElement(xPos) )
		poChild->SetLocalColor (eColor);
}


void Light::fn_vComputeSector ( void )
{
	MTH3D_tdstVector stVertex;
	GEO_tdxHandleToMatrix hAbsoluteMatrix = HIE_fn_hGetSuperObjectGlobalMatrix (GetSuperObject()->GetStruct());
	POS_fn_vGetTranslationVector( hAbsoluteMatrix, &stVertex );
	fn_vComputeSector (&stVertex);
}

ACP_tdxBool Light::fn_bSphericalLightAffectSector(HIE_tdxHandleToSuperObject hSuperSector)
{
	MTH3D_tdstVector			stMinPoint;
	MTH3D_tdstVector			stMaxPoint;
	MTH3D_tdstVector			stCenterPoint;
	MTH_tdxReal					xNear, xFar;
	SECT_tdxHandleOfSectorObject	hSectorObject;

	// compute border of sector object
	hSectorObject = (SECT_tdxHandleOfSectorObject) HIE_fn_hGetSuperObjectObject(hSuperSector);
	SECT_fn_vGetMaxPointInBorder(hSectorObject, &stMaxPoint);
	SECT_fn_vGetMinPointInBorder(hSectorObject, &stMinPoint);

	// check min & max point
	if ((stMinPoint.xX == MTH_C_InfinitPlus)  || 
		(stMinPoint.xY == MTH_C_InfinitPlus)  || 
		(stMinPoint.xZ == MTH_C_InfinitPlus)  || 
		(stMaxPoint.xX == MTH_C_InfinitMinus) || 
		(stMaxPoint.xY == MTH_C_InfinitMinus) || 
		(stMaxPoint.xZ == MTH_C_InfinitMinus))
		return FALSE;

	POS_fn_vGetTranslationVector( HIE_fn_hGetSuperObjectGlobalMatrix (GetSuperObject()->GetStruct()), &stCenterPoint );
	GLI_vGetLightNearFar((GLI_tdxHandleToLight)GetStruct(), &xNear, &xFar);
	if (INT_fn_bIntersectSphereWithBox(&stCenterPoint,xFar,&stMinPoint,&stMaxPoint))
		return TRUE;
	return FALSE;
}

void Light::fn_vComputeSector ( MTH3D_tdstVector* pstVertex )
{
	m_oSectorList.RemoveAll ();
	if (GetSuperObject()->GetModelName()==csSphericalModelName || GetSuperObject()->GetModelName()==csSpotModelName)
	{
		POSITION xSuperObjectPos;
		CPA_SuperObject* poSuperObject;
		CPA_List<CPA_SuperObject>* poList = GetInterface()->GetInterface()->GetObjectListByType	(C_szSectorTypeName);

		// current sector
		CPA_SuperObject* poCurrentSuperSector = GetInterface()->GetInterface()->GetSpecificInterface()->GetCurrentSectorFromPos (pstVertex);

		// for each sector
		for ( poSuperObject=poList->GetHeadElement(xSuperObjectPos); xSuperObjectPos; poSuperObject=poList->GetNextElement(xSuperObjectPos) )
		{
			if (fn_bIsSectorVisible  (poCurrentSuperSector, poSuperObject ) )
			{
				if (GetSuperObject()->GetModelName()==csSphericalModelName)
				{
					if (fn_bSphericalLightAffectSector(poSuperObject->GetStruct()))
						fn_vAddSector ( poSuperObject );
						
				}
				else
				{
					fn_vAddSector ( poSuperObject );
				}
			}
		}

	}
	else
	{
		CPA_SuperObject* poSuperSector = GetInterface()->GetInterface()->GetSpecificInterface()->GetCurrentSectorFromPos (pstVertex);
		fn_vAddSector ( poSuperSector );
	}
	fn_vNotifySave ();
}


void Light::fn_vUpdateLight (BOOL bComputeRLI,BOOL bUpdateScreen)
{
	CString csSectionName;

	// find referenced name of yhe light
	SCR_tdst_Link_Table* pLinkTable = LGT_fn_p_stGetLightLinkTable ();
	SCR_tdst_Link_Value* pLinkValue = SCR_fnp_st_Link_SearchValue(pLinkTable, (unsigned long) GetData());

	if ((pLinkValue)&&(SCR_M_e_Link_GetState(pLinkValue) == SCR_ELS_Link_Initialized))
		csSectionName = SCR_M_p_sz_Link_GetKey(pLinkValue);

	// compute name in .lg2 file
	char szFileName [SCR_CV_ui_Cfg_MaxLenName];
	char szActionName [SCR_CV_ui_Cfg_MaxLenName];
	char szName [SCR_CV_ui_Cfg_MaxLenName];
	char szSectionName [SCR_CV_ui_Cfg_MaxLenName];
	long lLenghtFileName;

	SCR_fn_v_RdL0_SplitSectionName((char*)(LPCSTR) csSectionName, szFileName, szActionName, szName);
	lLenghtFileName = strlen (szFileName);

	strcpy ( szFileName + lLenghtFileName - 3, c_Light_3DsMax_File_extension );
	SCR_fn_v_RdL0_ComputeSectionName ( szSectionName, szFileName, szActionName, szName);

	// engine light already in .lgt ?
	SCR_tdst_Link_Value* pValue = SCR_fnp_st_Link_SearchKey (pLinkTable, szSectionName);

	// light was not found in .lg2 -> impossible because 3DsMax flag is ON !!!
	ASSERT (pValue);

	// get engine of 3DsMax light
	GLI_tdxHandleToLight hEngineLight = (GLI_tdxHandleToLight) SCR_M_ul_Link_GetValue (pValue);

	// copy engine
	fn_vCopyLight ( (GLI_tdxHandleToLight)GetData(), hEngineLight );

	// set super object matrix
	GEO_tdxHandleToMatrix hMatrix = HIE_fn_hGetSuperObjectMatrix ( GetSuperObject()->GetStruct() );
	GLI_xGetLightMatrix ( hEngineLight, hMatrix ) ;

	// notify change
	fn_vNotifySave ();

	// set flag modification
	fn_vSetModify (TRUE);

	// compute RLI
	if(bComputeRLI) GetInterface()->fn_vComputeRLI ();

	// update screen
	if (bUpdateScreen) GetInterface()->GetInterface()->fn_vUpdateAll (E_mc_JustDraw);
}


BOOL Light::fn_bIsSectorVisible ( CPA_SuperObject *pSector, CPA_SuperObject *pIsVisibleSector)
{
	SECT_tdxHandleOfElementLstGraphicInteraction hElem;

	// no sector => no visibility
	if (!pIsVisibleSector)
		return FALSE;

	// current sector is visible
	if (pSector == pIsVisibleSector)
		return TRUE;

	// init engine struct
	HIE_tdxHandleToSuperObject hSector = pSector->GetStruct();
	HIE_tdxHandleToSuperObject hIsVisibleSector = pIsVisibleSector->GetStruct();
	// look in graphic list
	hElem = SECT_fn_hGetFirstGraphicNodeInGraphicList(hSector);
	while (hElem)
	{
		if (SECT_GetSectorInGraphicList(hElem) == hIsVisibleSector)
			return TRUE;
		hElem = SECT_fn_hGetNextGraphicNodeInGraphicList(hElem);
	}

	// sector is not visible
	return FALSE;
}



// draw the symbolical edition only when near far parametres have changed
void Light::fn_vSetNearFar ( MTH_tdxReal xNear, MTH_tdxReal xFar )
{
	GLI_tdxHandleToLight hLight = (GLI_tdxHandleToLight) GetEngineStruct ();
	GLI_vSetLightNearFar ( hLight, xNear, xFar );

	if (GetSuperObject()->GetModelName()==csSphericalModelName)
	{
		if (m_poNearSuperObject)
			((Geometry3D*)(m_poNearSuperObject->GetObject()))->fn_eSetRadius(&xNear, 0, 0);

		if (m_poFarSuperObject)
			((Geometry3D*)(m_poFarSuperObject->GetObject()))->fn_eSetRadius(&xFar, 0, 0);
	}
	else if (GetSuperObject()->GetModelName()==csSpotModelName)
	{
		MTH_tdxReal xLittleAlpha, xBigAlpha;
		MTH_tdxReal xConeRadius;

		GLI_vGetLightAlphas ( hLight, &xLittleAlpha, &xBigAlpha );


		if (m_poNearSuperObject)
		{
			MTH3D_tdstVector a2_stPoint [2];

			MTH3D_M_vSetVectorElements (a2_stPoint, 0.0, 0.0, 0.0 );
			MTH3D_M_vSetVectorElements (a2_stPoint+1, -xNear, 0., 0.);


			GEO_vSetListOfPointsOfObject
			(
				(ACP_tdxHandleOfObject)(HIE_fn_hGetSuperObjectObject(m_poNearSuperObject->GetStruct())),
				a2_stPoint,
				2,
				0
			);
			xConeRadius = (MTH_tdxReal)(xNear * tan (xLittleAlpha));
			((Geometry3D*)(m_poNearSuperObject->GetObject()))->fn_eSetRadius(&xConeRadius, 0, 0);

		}

		if (m_poFarSuperObject)
		{
			MTH3D_tdstVector a2_stPoint [2];

			MTH3D_M_vSetVectorElements (a2_stPoint, 0.0, 0.0, 0.0);
			MTH3D_M_vSetVectorElements (a2_stPoint+1, -xFar, 0., 0.);

			GEO_vSetListOfPointsOfObject
			(
				(ACP_tdxHandleOfObject)(HIE_fn_hGetSuperObjectObject(m_poFarSuperObject->GetStruct())),
				a2_stPoint,
				2,
				0
			);
			xConeRadius = (MTH_tdxReal)(xFar * tan (xBigAlpha));
			((Geometry3D*)(m_poFarSuperObject->GetObject()))->fn_eSetRadius(&xConeRadius, 0, 0);

		}
	}
}


BOOL Light::fn_vAddSector ( CPA_SuperObject* poObject )
{
	if (fn_bCheckEngineLimit (poObject))
	{
		m_oSectorList.AddTail (poObject);
		return TRUE;
	}
	else return FALSE;
}


BOOL Light::fn_bCheckEngineLimit ( CPA_SuperObject* poSuperObject )
{
	CPA_BaseObject* poBaseObject;
	Position xPos;
	long lNbSectorUse=0l;;

	CPA_BaseObjectList* poListOfLight = GetInterface()->GetMainWorld()->fn_p_oGetOriginalObjectList (C_szLightTypeName);

	for ( poBaseObject=poListOfLight->GetHeadElement(xPos); xPos; poBaseObject=poListOfLight->GetNextElement(xPos) )
	{
		Light* poLight = (Light*) poBaseObject;
		if (poLight->fn_bIsAffectedSector(poSuperObject)) lNbSectorUse++;
	}

	if (lNbSectorUse<C_lMaxLightInViewport) return TRUE;

	CString csMessage;
	CString csLimit;
	csLimit.Format ( "%hd", C_lMaxLightInViewport );
	csMessage = poSuperObject->GetObject()->GetName() + " is already used by " + csLimit + " lights.";
	MessageBeep (-1);
	M_GetMainWnd()->UpdateStatus( (char*) (LPCSTR) csMessage, C_STATUSPANE_INFOS, C_STATUS_WARNING );
	GetInterface()->fn_vSetClearStatus (TRUE);
	return FALSE;
}

//ROMTEAM color lights Ionut Grozea 10-08-1998
void  Light::SetMultiColor()
{

	GEO_tdxColorValue xRValue, xGValue, xBValue, xAValue;
	fn_vGetColor ( &xRValue, &xGValue, &xBValue, &xAValue );

	GMT_tdxHandleToGameMaterial hGameMaterial = GMT_fn_hCreateGameMaterial();
 	ACP_tdxHandleOfMaterial hMaterial;
 	GLI_xCreateMaterial ( &hMaterial ) ;
	GEO_tdstColor stColor;
	stColor.xR = xRValue;
	stColor.xG = xGValue;
	stColor.xB = xBValue;
	stColor.xA = 1.f;
	GLI_xSetMaterialAmbientCoef ( hMaterial, &stColor );
 
	GLI_xSetMaterialType ( hMaterial, C_lGouraudElement );
	GMT_fn_vSetVisualMaterial (hGameMaterial, hMaterial);

	CPA_SuperObjectBase* m_soBase = (CPA_SuperObjectBase*)GetSuperObject();

  	CList<CPA_EdElement<CPA_SuperObjectBase,HIE_tdstSuperObject_ *,HIE_tdstSuperObject_ *> *,CPA_EdElement<CPA_SuperObjectBase,HIE_tdstSuperObject_ *,HIE_tdstSuperObject_ *> *> *m_list = (CList<CPA_EdElement<CPA_SuperObjectBase,HIE_tdstSuperObject_ *,HIE_tdstSuperObject_ *> *,CPA_EdElement<CPA_SuperObjectBase,HIE_tdstSuperObject_ *,HIE_tdstSuperObject_ *> *>*)m_soBase;
	CPA_SuperObject* m_sObject = (CPA_SuperObject*)(m_list->GetHead());
 
// 	this->SetSaveStatus(E_ss_NoSave);
	CPA_BaseObject  * m_geomObject = m_sObject->GetObject();
    ((Geometry3D*)m_geomObject)->fn_vSetEngineGameMaterial( 0, 1, hGameMaterial);

}
void  Light::Set2Colors()
{

 
	GMT_tdxHandleToGameMaterial hGameMaterial = GMT_fn_hCreateGameMaterial();
 	ACP_tdxHandleOfMaterial hMaterial;
 	GLI_xCreateMaterial ( &hMaterial ) ;
	GEO_tdstColor stColor;
	if (fn_bGetOnOff ())
	{
		stColor.xR = 1.f;
		stColor.xG = 1.f;
		stColor.xB = 0.f;
		stColor.xA = 1.f;
	}
	else
	{
		stColor.xR = 0.f;
		stColor.xG = 1.f;
		stColor.xB = 1.f;
		stColor.xA = 1.f;
	}
	GLI_xSetMaterialAmbientCoef ( hMaterial, &stColor );
 
	GLI_xSetMaterialType ( hMaterial, C_lGouraudElement );
	GMT_fn_vSetVisualMaterial (hGameMaterial, hMaterial);

	CPA_SuperObjectBase* m_soBase = (CPA_SuperObjectBase*)GetSuperObject();

  	CList<CPA_EdElement<CPA_SuperObjectBase,HIE_tdstSuperObject_ *,HIE_tdstSuperObject_ *> *,CPA_EdElement<CPA_SuperObjectBase,HIE_tdstSuperObject_ *,HIE_tdstSuperObject_ *> *> *m_list = (CList<CPA_EdElement<CPA_SuperObjectBase,HIE_tdstSuperObject_ *,HIE_tdstSuperObject_ *> *,CPA_EdElement<CPA_SuperObjectBase,HIE_tdstSuperObject_ *,HIE_tdstSuperObject_ *> *>*)m_soBase;
	CPA_SuperObject* m_sObject = (CPA_SuperObject*)(m_list->GetHead());
 
// 	this->SetSaveStatus(E_ss_NoSave);
	CPA_BaseObject  * m_geomObject = m_sObject->GetObject();
    ((Geometry3D*)m_geomObject)->fn_vSetEngineGameMaterial( 0, 1, hGameMaterial);

}

//ROMTEAM color lights Ionut Grozea 10-08-1998

