/*
// ================================================================
// BitStream.cpp MMX VERSION 
// Ubi Sound Studios
// 18/03/1997
// Guillaume Michel
// ================================================================
*/

#include "SNDinc.h"

#if defined(_DLL_COMPILATION_MODE) 
#include "snddll.h"
#endif

#include "bitstrm.h"

// Size of the buffer (BYTES)
#define kSizeOfBuffer 4096
// Synch word
#define kSynchWord 0xfff 

void __cdecl getbits_(long l, void * P0, void* P2);
#define getbits	getbits_

//-------------------------------------------------------------------------
// Description : Initialise one bit stream
//-------------------------------------------------------------------------
// 
//-------------------------------------------------------------------------
// Input  : bitstream pointer
// Output : int
//-------------------------------------------------------------------------
// Creation date : 18/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------

SndBool SND_fn_bInitBitStream(SND_tdstBitStream* pBitStream)
{
	pBitStream->_bit_count = 0;
	pBitStream->_buf_pointer = 0;
	pBitStream->_end_buf = 0;
	pBitStream->_save_mm0 = 0;
	pBitStream->_save_mm0_2 = 0;
	pBitStream->_bOk = 0;

	pBitStream->ulDataOffset = 0;
	pBitStream->ulDolbyMark = 0;
	pBitStream->ulFinalSize = 0;
	return TRUE;
}

//-------------------------------------------------------------------------
// Description : allocate memory for one bitstream
//-------------------------------------------------------------------------
// ALLOCATE MEMORY
//-------------------------------------------------------------------------
// Input  : bitstream pointer
// Output : int
//-------------------------------------------------------------------------
// Creation date : 24/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------

SndBool SND_fn_bAllocateBitStream(SND_tdstBitStream* pBitStream)
{
	/*
	 * Allocate memory once at the begining of the program.
	 * In the case of a file bitstream we do not use this buffer
	 * kSizeOfBuffer = 4096
	 */
	pBitStream->pucBuffer = (unsigned char*) SND_fn_pvMallocSnd(kSizeOfBuffer);
	return ((pBitStream->pucBuffer) != NULL);
}

//-------------------------------------------------------------------------
// Description : free one bit stream
//-------------------------------------------------------------------------
// DO FREE MEMORY !!!
//-------------------------------------------------------------------------
// Input  : bitstream pointer
// Output : int
//-------------------------------------------------------------------------
// Creation date : 24/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------

SndBool SND_fn_bFreeBitStream(SND_tdstBitStream* pBitStream)
{
	SND_fn_bCloseBitStream(pBitStream);
	if (pBitStream->pucBuffer) SND_fn_vFreeSnd(pBitStream->pucBuffer);
	pBitStream->pucBuffer = NULL;
	return TRUE;
}

//-------------------------------------------------------------------------
// Description : Open a new bitstream
//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
// Input  : filename and bitstream pointer
// Output : int
//-------------------------------------------------------------------------
// Creation date : 18/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------

SndBool SND_fn_bOpenBitStreamFile(SND_tdstBitStream* pBitStream, char* pFilename, unsigned long ulOffset)
{
	long lTmp;
	char tmp[256];

	snd_assert(pBitStream);

	pBitStream->_bit_count = 0;
	pBitStream->_save_mm0 = 0;
	pBitStream->_save_mm0_2 = 0;
	pBitStream->_bOk = 0;

    SND_fn_vResolveFileName(pFilename,tmp);
	pBitStream->pFile = SND_fn_hOpenFileReadSnd(tmp);

	if (pBitStream->pFile==OPENFILESND_FAILED)
	{
		SND_fn_vDisplayError(E_uwSndCannotOpenFile,pFilename);
		return FALSE;
	}

	snd_assert(pBitStream->pFile!=OPENFILESND_FAILED);
	SND_fn_dwSeekFileSnd(pBitStream->pFile,ulOffset,SEEKFILESND_BEGIN);
	
	lTmp = SND_fn_dwReadFileSnd(pBitStream->pFile,kSizeOfBuffer,pBitStream->pucBuffer);
	pBitStream->_buf_pointer = (long)(pBitStream->pucBuffer);
	pBitStream->_end_buf = (long)((pBitStream->pucBuffer)+lTmp);

	pBitStream->ulFinalSize = *((long*)(pBitStream->pucBuffer));
	getbits(32,&lTmp,pBitStream);	// SKIP THE FIRST 32 BITS
	
	pBitStream->ulDolbyMark = *((long*)(pBitStream->pucBuffer+4));
	if ((pBitStream->ulDolbyMark == 'SUR1') || (pBitStream->ulDolbyMark == 'SUR2')){
		// the file is surround
		getbits(32,&lTmp,pBitStream);	// SKIP THE FIRST 32 BITS
	}

	return (pBitStream->_bOk == 0);
}

//-------------------------------------------------------------------------
// Description : Open a new bitstream
//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
// Input  : bitstream pointer and buffer and size
// Output : int
//-------------------------------------------------------------------------
// Creation date : 18/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------

SndBool SND_fn_bOpenBitStreamMem(SND_tdstBitStream* pBitStream,unsigned char* pBuffer, unsigned long ulSize)
{	
	long lTmp;

	// IN THE MMX VERSION, THE SIZE MUST BE 8 BYTE ALIGNED
	// ULSIZE % 8  = 0

	pBitStream->_bit_count = 0;
	pBitStream->_save_mm0 = 0;
	pBitStream->_save_mm0_2 = 0;
	pBitStream->_bOk = 0;

	pBitStream->_buf_pointer = (long)pBuffer;
	pBitStream->_end_buf = (long)(pBuffer+ulSize);

	pBitStream->ulFinalSize = *((long*)pBuffer);
	getbits(32,&lTmp,pBitStream);	// SKIP THE FIRST 32 BITS

	pBitStream->ulDolbyMark = *((long*)pBuffer+1);

	if ((pBitStream->ulDolbyMark == 'SUR1') || (pBitStream->ulDolbyMark == 'SUR2')){
		// The file is Surround
		getbits(32,&lTmp,pBitStream);	// SKIP THE FIRST 32 BITS
	}

	return (pBitStream->_bOk == 0);
}

//-------------------------------------------------------------------------
// Description : Free a bitstream
//-------------------------------------------------------------------------
// DOES NOT FREE MEMORY !!!!
//-------------------------------------------------------------------------
// Input  : bitstream pointer
// Output : int
//-------------------------------------------------------------------------
// Creation date : 18/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------

SndBool SND_fn_bCloseBitStream(SND_tdstBitStream* pBitStream)
{

	if (pBitStream->pFile!=OPENFILESND_FAILED)	{
		SND_fn_vCloseFileSnd(pBitStream->pFile);
		pBitStream->pFile = OPENFILESND_FAILED;
	}

	pBitStream->_bit_count = 0;
	pBitStream->_buf_pointer = 0;
	pBitStream->_end_buf = 0;
	pBitStream->_save_mm0 = 0;
	pBitStream->_save_mm0_2 = 0;
	pBitStream->_bOk = 0;
	return TRUE;

}

//-------------------------------------------------------------------------
// Description : read one bit from the bitstream
//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
// Input  : bitstream pointer
//			pointer on value we want to fill
// Output : int
//-------------------------------------------------------------------------
// Creation date : 18/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------
/*
SndBool SND_fn_bGet1Bit(SND_tdstBitStream* pBitStream, unsigned short *puwValue)
{
	getbits(1,puwValue,pBitStream);
	return (pBitStream->_bOk==0);
}
*/
//-------------------------------------------------------------------------
// Description : read many bits from the bitstream
//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
// Input  : bitstream pointer
//			pointer on value we want to fill
//			number of bits we want to read... ( <= 16)
// Output : int
//-------------------------------------------------------------------------
// Creation date : 18/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------
/*
SndBool SND_fn_bGetNBits(SND_tdstBitStream* pBitStream,unsigned short *puwValue, unsigned char ucNbBits)
{
	getbits(ucNbBits,puwValue,pBitStream);
	return (pBitStream->_bOk==0);
}
*/
//-------------------------------------------------------------------------
// Description : seek the synch word
//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
// Input  : bitstream pointer
// Output : int TRUE if found
//-------------------------------------------------------------------------
// Creation date : 18/03/97              Author : Guillaume Michel
//-------------------------------------------------------------------------
// Modifications :
// Modification Date :              Author :
//-------------------------------------------------------------------------

SndBool SND_fn_bSeekSynch(SND_tdstBitStream* pBitStream)
{
	unsigned short uwSynch = 0;
	unsigned short uwTmp;
	int bFound = FALSE;

	getbits(12,&uwSynch,pBitStream);
	if (pBitStream->_bOk == 0) {
		if (uwSynch == kSynchWord) return TRUE;
		// We must search again...
		while(!bFound) {
			uwSynch <<= 1;
			getbits(1,&uwTmp,pBitStream);
			if (pBitStream->_bOk == 1)
				return FALSE; // End of file...

			uwSynch |= uwTmp;
			uwSynch &= 0xfff;
			bFound = (uwSynch == kSynchWord);
		}

		// We have found the synch word...
		return TRUE;

	} else return FALSE;
} 

void  __stdcall refill_buffer(SND_tdstBitStream* pBitStream)
{
	long lTmp;
	if (pBitStream->pFile!=OPENFILESND_FAILED) {
		// File BitStream
		lTmp = SND_fn_dwReadFileSnd(pBitStream->pFile,kSizeOfBuffer,pBitStream->pucBuffer);
		pBitStream->_buf_pointer = (long)(pBitStream->pucBuffer);
		pBitStream->_end_buf = (long)((pBitStream->pucBuffer)+lTmp);
		pBitStream->_bOk = (lTmp == 0)?1:0;
	} else {
		// Memory BitStream
		pBitStream->_bOk = 1; // finished
	}
}
